#!/usr/local/bin/perl 

$| = 1;

use LWP::Simple;
use CGI qw(:all);
use CGI::Carp qw(fatalsToBrowser);

# Copyright (c) 2007 The Regents of the University of California.
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions
# are met:
#   1. Redistributions of source code must retain the above copyright
#      notice, this list of conditions, and the following disclaimer.
#   2. Redistributions in binary form must reproduce the above
#      copyright notice, this list of conditions, and the following
#      disclaimer in the documentation and/or other materials provided
#      with the distribution.
#   3. Redistributions must acknowledge that this software was
#      originally developed by the UCSF Computer Graphics Laboratory
#      under support by the NIH National Center for Research Resources,
#      grant P41-RR01081.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDER "AS IS" AND ANY
# EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
# IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
# PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE REGENTS BE LIABLE
# FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
# CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT
# OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR
# BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
# WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE
# OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
# EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

$analyze = new CGI;
$analtype = param('analtype');

#program assignments
#change TMPDIR_raw to the complete directory path of your tmp3 location
$TMPDIR_raw = "/usr/tmp/open";
#change topo2run to your complete directory path to the TOPO2 python source
#code on your server
$topo2run = "/usr/local/src/www/sacs/htdocs/TOPO2/cgi-bin";
#change local_lib to the complete directory path for the location of the
#required TMHMM files on your server
$local_lib = "$topo2run";
#change local_bin to the complete directory path for the TMHMM decodeanhmm 
#program on your server
$local_bin = "$topo2run";

$decodeanhmm = "$local_bin/decodeanhmm"; 
$options = "$local_lib/webTMHMM2.0.options"; 
$options2 = "$local_lib/TMHMM2.0.options"; 
$model = "$local_lib/TMHMM2.0.model";
 
$topow = "$topo2run/topo2.py";
$topops = "$topo2run/topops.py";
$topopng = "$topo2run/topoparser.py";

# NO CHANGES SHOULD BE NEEDED BEYONG THIS POINT
#=======================================================================

#draw form for submission of database sequence searching
print $analyze->header();
print $analyze->start_html(-title=>'Analyze',
			-bgcolor=>'#ffff66');
print $analyze->h1("Analysis Result for TMHMM"),p;

#give each project a unique name
if (param('seqid')){
        $seqnam = param('seqid');
        $seqnam =~ s/\s+//g; 
        $seqid = $seqnam."_$$"; 
}else{
	$seqid = "seqid"."_$$";
}

open (TMMDIR, "|mkdir $TMPDIR_raw/$seqid");
close TMMDIR;

$TMPDIR = "$TMPDIR_raw/$seqid";


$analyzescript = join ("-","$seqid","analyzescript");

#make sure form is properly filled out
if (param('sequence')&&param('sequencefile')){
        print h1(blink("Enter only one sequence per analysis.")),p;
        exit;
}elsif(param('sequence')){
        $sequence = param('sequence');
}elsif(param('sequencefile')){
        $sequence = param('sequencefile');
}else{
        print h1(blink("You must enter a sequence for an analysis.")),p;
        exit;
}

# Decide what input type to use. Paste in sequence or Accession number.
#$seqtype = param('seqtype');

if (param('seqtype') eq 'pastein'){

        $sequence =~ tr/a-zA-Z//cd;
        $sequence =~ tr/a-z/A-Z/;
        #write out sequence file for reformatting
        open (SEQFILE, ">$TMPDIR/$seqid");
        print SEQFILE ">$seqid\n$sequence\n";
        close (SEQFILE);
        `cp $TMPDIR/$seqid $TMPDIR/$seqid-x`;

}else{

	$url="http://www.ncbi.nlm.nih.gov/htbin-post/Entrez/query?db=p&form=6&Dopt=f&html=no&uid=$sequence";

        @entry = split(/\n/,(get $url));

    if ($entry[2] =~/^Entrez Reports/ ||
        $entry[0] =~/^ERROR/) {

        print "$seq_in ERROR";

        print "\n<tt><pre>$url</tt></pre>\n";
        print "\n<h2>ERROR - $seq_in not found</h2>\n";
        exit;
    }else {

	open (SEQFILE, ">$TMPDIR/$seqid");
        $have_gt = 0;
        for $l (@entry) {
            if ($l=~/^>/) {$have_gt = 1;}
            if ($have_gt == 1) { print SEQFILE "$l\n";}
        }
        close (SEQFILE);
	open (SEQFILE, "$TMPDIR/$seqid");
        open (SEQFILE2, ">$TMPDIR/$seqid-x");
        $i = 1;
        while (<SEQFILE>){
           if($i == 1){
              $line = ">"."$seqid"." sequence from the database";
              print SEQFILE2 "$line\n";
              $i = $i + 1;
           }else{
              print SEQFILE2 "$_";
           }
        }
        close (SEQFILE);
        close (SEQFILE2);
    }
}

$seqidout = "$seqid"."-tmhmm";
$seqidout2 = "$seqidout"."-2";
$seqidout3 = "$seqidout"."-3";

open (SCRIPT, ">$TMPDIR/$analyzescript");
print SCRIPT <<"END OF SCRIPT";
cat $TMPDIR/$seqid | $decodeanhmm -f $options -modelfile $model >$TMPDIR/$seqidout
cat $TMPDIR/$seqid | $decodeanhmm -f $options2 -modelfile $model >$TMPDIR/$seqidout2
END OF SCRIPT
close (SCRIPT);

#run CSH script
system("/bin/csh $TMPDIR/$analyzescript");
#`rm -f $TMPDIR/$analyzescript`;
`wait 10`;

#process the output file
open (OUTS,"$TMPDIR/$seqidout");
$i = 0;
while (<OUTS>){
   $line = $_;
   $line = $_;
   @fields = split(' ',$line);
   $lenf = @fields;
   #check for prediction
   if($lenf > 5){
	if($fields[2] ne '0'){
	     if($fields[2] eq 'i'){
		$where_start = 2;
	     }elsif($fields[2] eq 'I'){
		$where_start = 2;
             }elsif($fields[2] eq 'o'){
		$where_start = 1;
             }elsif($fields[2] eq 'O'){
		$where_start = 1;
             }
	     $longs = length($line);
	     $m1 = index($line,'M');
	     $line2 = $line;
	     while($m1 > 0){
		$longs2 = length($line2);
	     	$m1 = index($line2,'M');
	        $part = substr($line2,$m1+2,$longs2-($m1+2));
		#print "the value is -$part-",br; 
		$spc = index($part,' ');
		$comm = index($part,',');
		$p1 = substr($part,0,$spc);
		$p2 = substr($part,$spc+1,$comm-($spc+1));
 	        if($i == 0){
		   #print "the values are $p1 and $p2",br;
	           $b1 = $p1;
	           $e1 = $p2;
 	        }elsif($i == 1){
		   #print "the values are $p1 and $p2",br;
	           $b2 = $p1;
	           $e2 = $p2;
 	        }elsif($i == 2){
		   #print "the values are $p1 and $p2",br;
	           $b3 = $p1;
	           $e3 = $p2;
 	        }elsif($i == 3){
		   #print "the values are $p1 and $p2",br;
	           $b4 = $p1;
	           $e4 = $p2;
 	        }elsif($i == 4){
	           $b5 = $p1;
	           $e5 = $p2;
 	        }elsif($i == 5){
	           $b6 = $p1;
	           $e6 = $p2;
 	        }elsif($i == 6){
	           $b7 = $p1;
	           $e7 = $p2;
 	        }elsif($i == 7){
	           $b8 = $p1;
	           $e8 = $p2;
 	        }elsif($i == 8){
	           $b9 = $p1;
	           $e9 = $p2;
 	        }elsif($i == 9){
	           $b10 = $p1;
	           $e10 = $p2;
 	        }elsif($i == 10){
	           $b11 = $p1;
	           $e11 = $p2;
 	        }elsif($i == 11){
	           $b12 = $p1;
	           $e12 = $p2;
 	        }elsif($i == 12){
	           $b13 = $p1;
	           $e13 = $p2;
 	        }elsif($i == 13){
	           $b14 = $p1;
	           $e14 = $p2;
 	        }elsif($i == 14){
	           $b15 = $p1;
	           $e15 = $p2;
 	        }elsif($i == 15){
	           $b16 = $p1;
	           $e16 = $p2;
 	        }elsif($i == 16){
	           $b17 = $p1;
	           $e17 = $p2;
 	        }elsif($i == 17){
	           $b18 = $p1;
	           $e18 = $p2;
 	        }elsif($i == 18){
	           $b19 = $p1;
	           $e19 = $p2;
 	        }elsif($i == 19){
	           $b20 = $p1;
	           $e20 = $p2;
 	        }elsif($i == 20){
	           $b21 = $p1;
	           $e21 = $p2;
 	        }elsif($i == 21){
	           $b22 = $p1;
	           $e22 = $p2;
 	        }elsif($i == 22){
	           $b23 = $p1;
	           $e23 = $p2;
 	        }elsif($i == 23){
	           $b24 = $p1;
	           $e24 = $p2;
 	        }elsif($i == 24){
	           $b25 = $p1;
	           $e25 = $p2;
 	        }elsif($i == 25){
	           $b26 = $p1;
	           $e26 = $p2;
 	        }elsif($i == 26){
	           $b27 = $p1;
	           $e27 = $p2;
 	        }elsif($i == 27){
	           $b28 = $p1;
	           $e28 = $p2;
 	        }elsif($i == 28){
	           $b29 = $p1;
	           $e29 = $p2;
 	        }elsif($i == 29){
	           $b30 = $p1;
	           $e30 = $p2;
 	        }elsif($i == 30){
	           $b31 = $p1;
	           $e31 = $p2;
 	        }elsif($i == 31){
	           $b32 = $p1;
	           $e32 = $p2;
 	        }elsif($i == 32){
	           $b33 = $p1;
	           $e33 = $p2;
 	        }elsif($i == 33){
	           $b34 = $p1;
	           $e34 = $p2;
 	        }elsif($i == 34){
	           $b35 = $p1;
	           $e35 = $p2;
 	        }elsif($i == 35){
	           $b36 = $p1;
	           $e36 = $p2;
 	        }elsif($i == 36){
	           $b37 = $p1;
	           $e37 = $p2;
 	        }elsif($i == 37){
	           $b38 = $p1;
	           $e38 = $p2;
 	        }elsif($i == 38){
	           $b39 = $p1;
	           $e39 = $p2;
 	        }elsif($i == 39){
	           $b40 = $p1;
	           $e40 = $p2;
 	        }elsif($i == 40){
	           $b41 = $p1;
	           $e41 = $p2;
 	        }elsif($i == 41){
	           $b42 = $p1;
	           $e42 = $p2;
 	        }elsif($i == 42){
	           $b43 = $p1;
	           $e43 = $p2;
 	        }elsif($i == 43){
	           $b44 = $p1;
	           $e44 = $p2;
 	        }elsif($i == 44){
	           $b45 = $p1;
	           $e45 = $p2;
 	        }elsif($i == 45){
	           $b46 = $p1;
	           $e46 = $p2;
 	        }elsif($i == 46){
	           $b47 = $p1;
	           $e47 = $p2;
 	        }elsif($i == 47){
	           $b48 = $p1;
	           $e48 = $p2;
 	        }elsif($i == 48){
	           $b49 = $p1;
	           $e49 = $p2;
 	        }elsif($i == 49){
	           $b50 = $p1;
	           $e50 = $p2;
 	        }elsif($i == 50){
	           $b51 = $p1;
	           $e51 = $p2;
 	        }elsif($i == 51){
	           $b52 = $p1;
	           $e52 = $p2;
 	        }elsif($i == 52){
	           $b53 = $p1;
	           $e53 = $p2;
 	        }elsif($i == 53){
	           $b54 = $p1;
	           $e54 = $p2;
 	        }elsif($i == 54){
	           $b55 = $p1;
	           $e55 = $p2;
 	        }elsif($i == 55){
	           $b56 = $p1;
	           $e56 = $p2;
 	        }elsif($i == 56){
	           $b57 = $p1;
	           $e57 = $p2;
 	        }elsif($i == 57){
	           $b58 = $p1;
	           $e58 = $p2;
 	        }elsif($i == 58){
	           $b59 = $p1;
	           $e59 = $p2;
 	        }elsif($i == 59){
	           $b60 = $p1;
	           $e60 = $p2;
                }
		$longs2 = length($part);
		$linex = substr($part,$comm+1,$longs2-($comm+1));
		$line2 = $linex;
		#print "the line is -$line2-",br;
	     	$m1 = index($line2,'M');
		$i = $i + 1;
            }
        } 
   }
}
close (OUTS);
$nseg = $i;

## get data for topo run
if($nseg == 0){
   print "<b>TMHMM does not predict this to be a transmembrane protein</b>",p;
}elsif($nseg > 0){
# other default TOPO parameters
   $topoparms = "$seqid"."-topoparms";
   $toposcript = "$seqid"."-toposcript";
   $seqidin = "$seqid".".pep";

## write out parameters to a file
   open (STUFF, ">$TMPDIR/$topoparms")|| die "couldn't open parms: ";
   $lix = join("\t","$seqid","$nseg","$where_start","UNKNOWN");
   print STUFF "$lix\n";
   $i = 1;
   while ($i <= $nseg){
      if($i == 1){
         $name = $b1;
         $name2 = $e1;
      }elsif($i == 2){
         $name = $b2;
         $name2 = $e2;
      }elsif($i == 3){
         $name = $b3;
         $name2 = $e3;
      }elsif($i == 4){
         $name = $b4;
         $name2 = $e4;
      }elsif($i == 5){
         $name = $b5;
         $name2 = $e5;
      }elsif($i == 6){
         $name = $b6;
         $name2 = $e6;
      }elsif($i == 7){
         $name = $b7;
         $name2 = $e7;
      }elsif($i == 8){
         $name = $b8;
         $name2 = $e8;
      }elsif($i == 9){
         $name = $b9;
         $name2 = $e9;
      }elsif($i == 10){
         $name = $b10;
         $name2 = $e10;
      }elsif($i == 11){
         $name = $b11;
         $name2 = $e11;
      }elsif($i == 12){
         $name = $b12;
         $name2 = $e12;
      }elsif($i == 13){
         $name = $b13;
         $name2 = $e13;
      }elsif($i == 14){
         $name = $b14;
         $name2 = $e14;
      }elsif($i == 15){
         $name = $b15;
         $name2 = $e15;
      }elsif($i == 16){
         $name = $b16;
         $name2 = $e16;
      }elsif($i == 17){
         $name = $b17;
         $name2 = $e17;
      }elsif($i == 18){
         $name = $b18;
         $name2 = $e18;
      }elsif($i == 19){
         $name = $b19;
         $name2 = $e19;
      }elsif($i == 20){
         $name = $b20;
         $name2 = $e20;
      }elsif($i == 21){
         $name = $b21;
         $name2 = $e21;
      }elsif($i == 21){
         $name = $b21;
         $name2 = $e21;
      }elsif($i == 22){
         $name = $b22;
         $name2 = $e22;
      }elsif($i == 23){
         $name = $b23;
         $name2 = $e23;
      }elsif($i == 24){
         $name = $b24;
         $name2 = $e24;
      }elsif($i == 25){
         $name = $b25;
         $name2 = $e25;
      }elsif($i == 26){
         $name = $b26;
         $name2 = $e26;
      }elsif($i == 27){
         $name = $b27;
         $name2 = $e27;
      }elsif($i == 28){
         $name = $b28;
         $name2 = $e28;
      }elsif($i == 29){
         $name = $b29;
         $name2 = $e29;
      }elsif($i == 30){
         $name = $b30;
         $name2 = $e30;
      }elsif($i == 31){
         $name = $b31;
         $name2 = $e31;
      }elsif($i == 32){
         $name = $b32;
         $name2 = $e32;
      }elsif($i == 33){
         $name = $b33;
         $name2 = $e33;
      }elsif($i == 34){
         $name = $b34;
         $name2 = $e34;
      }elsif($i == 35){
         $name = $b35;
         $name2 = $e35;
      }elsif($i == 36){
         $name = $b36;
         $name2 = $e36;
      }elsif($i == 37){
         $name = $b37;
         $name2 = $e37;
      }elsif($i == 38){
         $name = $b38;
         $name2 = $e38;
      }elsif($i == 39){
         $name = $b39;
         $name2 = $e39;
      }elsif($i == 40){
         $name = $b40;
         $name2 = $e40;
      }elsif($i == 41){
         $name = $b41;
         $name2 = $e41;
      }elsif($i == 42){
         $name = $b42;
         $name2 = $e42;
      }elsif($i == 43){
         $name = $b43;
         $name2 = $e43;
      }elsif($i == 44){
         $name = $b44;
         $name2 = $e44;
      }elsif($i == 45){
         $name = $b45;
         $name2 = $e45;
      }elsif($i == 46){
         $name = $b46;
         $name2 = $e46;
      }elsif($i == 47){
         $name = $b47;
         $name2 = $e47;
      }elsif($i == 48){
         $name = $b48;
         $name2 = $e48;
      }elsif($i == 49){
         $name = $b49;
         $name2 = $e49;
      }elsif($i == 50){
         $name = $b50;
         $name2 = $e50;
      }elsif($i == 51){
         $name = $b51;
         $name2 = $e51;
      }elsif($i == 52){
         $name = $b52;
         $name2 = $e52;
      }elsif($i == 53){
         $name = $b53;
         $name2 = $e53;
      }elsif($i == 54){
         $name = $b54;
         $name2 = $e54;
      }elsif($i == 55){
         $name = $b55;
         $name2 = $e55;
      }elsif($i == 56){
         $name = $b56;
         $name2 = $e56;
      }elsif($i == 57){
         $name = $b57;
         $name2 = $e57;
      }elsif($i == 58){
         $name = $b58;
         $name2 = $e58;
      }elsif($i == 59){
         $name = $b59;
         $name2 = $e59;
      }elsif($i == 60){
         $name = $b60;
         $name2 = $e60;
      }
      $lix2 = join("\t","$name","$name2","1");
      print STUFF "$lix2\n";
      $i = $i +1;
   }
}
$ii = 0;
while ($ii < 4){
   $lix = "n";
   print STUFF "$lix\n";
   $ii = $ii + 1
}
close(STUFF);

open (ANALYSISRESULT, "$TMPDIR/$seqidout2") || die "Cannot open result from hmmtop run\n";
while (<ANALYSISRESULT>){
        push @analysisresult, $_;
}
print $analyze->pre(@analysisresult);

$outstopo = join(".","$seqid","xml");
$outstopo2 = join(".","$seqid","ps");
if($nseg > 0){
   open (SCRIPT, ">$TMPDIR/$toposcript");
   print SCRIPT <<"END OF SCRIPT";
cd $TMPDIR
$topow  -o $outstopo $topoparms
$topops -o $outstopo2 $outstopo
$topopng $TMPDIR/$outstopo
END OF SCRIPT
   close (SCRIPT);


##run CSH script
   system ("/bin/csh $TMPDIR/$toposcript");
   `wait 10`;

$outstopo6 = 'test.png';
#check to see what sort of image output was created
`ls  $TMPDIR/$seqid/$outstopo6 > $TMPDIR/$seqid/img-hit`;
$imgct = 0;
open (IMGCHECK,"$TMPDIR/$seqid/img-hit");
while (<IMGCHECK>){
    $imgct = $imgct + 1;
}
close (IMGCHECK);

   p;
   print "<hr>";
   print "<b>Basic TOPO2 output image of the previous prediction.</b>",p;
   p;
   if($imgct > 0){
           print "<img src=\"/tmp3/$seqid/test.png\"><p>";
   }else{
           print "<img src=\"/tmp3/$seqid/test.gif\"><p>";
   }
   p;
   print " ",a({href=>"/tmp3/$seqid/$outstopo2"},'postscript data file'),p;

}

#clean up un-necessary files
`rm -f $TMPDIR/$seqid`;
#`rm -f $TMPDIR/$seqid-tmhmm*`;
`rm -f $TMPDIR/$seqid-a*`;
#`rm -f $TMPDIR/$seqid-topo*`;
`rm -f $TMPDIR/$seqid-x`;
`rm -f $TMPDIR/$seqid.xml`;

1;

p;
print hr();
p;
print  "Sometimes clicking on the browser BACK button will return you to the
submission page with your data intact.",p;

print $analyze->end_html;



