# Copyright (c) 2007 The Regents of the University of California.
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions
# are met:
#   1. Redistributions of source code must retain the above copyright
#      notice, this list of conditions, and the following disclaimer.
#   2. Redistributions in binary form must reproduce the above
#      copyright notice, this list of conditions, and the following
#      disclaimer in the documentation and/or other materials provided
#      with the distribution.
#   3. Redistributions must acknowledge that this software was
#      originally developed by the UCSF Computer Graphics Laboratory
#      under support by the NIH National Center for Research Resources,
#      grant P41-RR01081.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDER "AS IS" AND ANY
# EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
# IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
# PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE REGENTS BE LIABLE
# FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
# CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT
# OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR
# BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
# WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE
# OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
# EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

from Image import *
from Tools import *
from ZB import *

class Loop:
	"a class for loop information"
	def __init__(self,num,start,end,type,side):
		self.numLoop = 0
		self.startLoop = 0
		self.endLoop = 0
		self.typeLoop = 0
		self.sideLoop = 0
		self.numLoop = int(num)
		self.startLoop = int(start)
		self.endLoop = int(end)
		self.typeLoop = int(type)
		self.sideLoop = int(side)

		return

def drawLP(r,CD,x,whichside,flip,verbose):
	#determine the loop values

	end_prob = 0
	#parameters for the loop
	#N - number of residues in the chain to be displayed  
	#W - width between the centers of transmembrane segments
	#R - symbol radius 
	#print "-|into the loop drawing section|-"

	l5 = 1
	R = 5.0
	W = 16.0*R
	B = 1
	sl = 3.5

	if whichside == 1:
		ybase = 0.0 + R
		fact = -1.0
	else:
		ybase = 0.0 - (2.0*(B + .5*sl) + 5.0*(sl + B) + 1)*R
		fact = 1.0

	ybaseb = ybase + 4.0*R

	custom = 0
	# determine if the loop is to be customized
	if r.cusdo == "y":
		#print "checking for a custom loop"
		cuschk = 0
		stops = len(r.seq)
		while cuschk < r.cusnum:
			#print "cuslook is %d" %r.cusloop[cuschk]
			
			cus_make = r.cusloop[cuschk] - 1

			if cus_make == CD.numCD:
				custom = 1
				msg = "cus make is %d and loop num is %d" %(cus_make,CD.numCD)
				printVerbose(verbose,msg)
				cusidnum = cuschk
				break
			cuschk = cuschk + 1


	#determine the size of the loop
	N = CD.clenCD	
	if N == 0:
		msg = "there is a zero bridge here"
		printVerbose(verbose,msg)
		msg = "x is %f and whichside is %s" %(x,whichside)
		printVerbose(verbose,msg)
		makebridge(r,x,whichside,verbose)

	elif N <= 100:
		chain = 10
	else:
		chain = 30

	if flip == 1:
		dir = -1.0
	else:
		dir = 1.0


	#print "whichside is %d and num is %d" % (whichside,CD.numCD)

	if CD.numCD == 0 or CD.numCD == 99:
		if CD.numCD == 99:
			if r.farend > x:
				end_prob = 1
				msg = "found an ending problem"
				printVerbose(verbose,msg)

		if custom == 1:
			print "working with a custom end"

		if whichside == 2:
			fact = -1.0*fact
			ybase = ybase -2.0*R

		#this is an open loop at the beginning of the structure
		if CD.numCD == 0:
			rev = reverse(CD.strandCD)

		#if whichside == 1:
			#print "whichside is 1 and flip is %d" % flip

		#start is at the top
		#determine the number of looping parts
		lop = (N-2)/(chain+2)
		lox = (N-2)%(chain+2.0)
		if lox > 0:
			lop = lop + 1

		#print rev

		ybaset = ybaseb + chain*2.0*R

		#draw first two symbols
		k = 0
		
		n = 1
		while n <=2:
			if whichside == 2:
				m = 2-n
			else:
				m = n-1

			y = ybase - (m)*2.0*R*fact
			if CD.numCD == 99:
				l1 = CD.startCD + k
                                l2 = CD.strandCD[k:k+1]
				if l1 != 0:
					loop_image_data3(r,k,CD,l1,l2,x,y,whichside,verbose)
			else:
				l1 = CD.endCD - k
	                        l2 = rev[k:k+1]
				if l1 != 0:
					loop_image_data2(r,CD,l1,l2,x,y,whichside,verbose)

			n = n + 1
			k = k +1
			if k == N:
				break


		j = 1
                if custom == 0 and end_prob == 1 and CD.numCD == 99:
                    	#print "into the new section"
			chain2 = 9
			if whichside == 2:			
				ybase2 = y - 2.0*R
				ybaset2 = y
				ybaset = ybase2
				ybaseb = y -(chain+1)*2.0*R
				ybaseb2 = y -(chain2+1)*2.0*R

			if whichside == 1:
	               		ybaset2 = ybaseb + chain2*2.0*R

			while j <= 4:
				#check to see which leg you are on
				one = j/2
				two = j%2

				if two == 0:
					factors = 1.0
				else:
					factors = -1.0

				if CD.numCD == 99:
					factors = -1.0*factors


				l = 1
				while l <= chain2:
					if whichside == 2:
						if j == 1:
							m = l-1
						else:
							m = l

					elif whichside == 1:
						if two != 0:
							m = l-1
							ybase2 = ybaseb
						elif two == 0:
							m = l
							ybase2 = ybaset2 
						
					y = ybase2 - (m)*2.0*R*fact*factors*dir
					l1 = CD.startCD + k
                                	l2 = CD.strandCD[k:k+1]
					loop_image_data3(r,k,CD,l1,l2,x,y,whichside,verbose)

					l = l + 1
					if k == N:
						#print "ending this"
						#print "j is %d and k is %d" % (j,k)
						break
					k = k + 1
				if k == N:
					break
				if whichside == 1:
					ybase2 = y
				else:
					ybase2 = y + 2.0*R

				#draw cross over section
				n = 1
				x = x + R

				while n <=2:
					if whichside == 1:
						if two != 0:
							m = 0
							ybase2 = ybaset2
						else:
							m = 1
							ybase2 = ybase2
					else:
						m = 0
						if two != 0:	
							ybase2 = ybaseb2
						else:
							ybase2 = ybaset2

					y = ybase2 - (m)*2.0*R*factors*fact*dir
					l1 = CD.startCD + k
       	                       		l2 = CD.strandCD[k:k+1]
					loop_image_data3(r,k,CD,l1,l2,x,y,whichside,verbose)
					n = n + 1
					if k == N:
						break
					k = k +1
				
					if CD.numCD == 99:
						x = x + 2.0*R
						y = y + 2.0*R 
				if k == N:
					break

				j = j + 1
				if CD.numCD == 99:
					x = x - R
			j = 1
			while j <= lop-1:
				#check to see which leg you are on
				one = j/2
				two = j%2
				if two == 0:
					factors = 1.0
				else:
					factors = -1.0

				if CD.numCD == 99:
					factors = -1.0*factors

				if whichside == 2:			
					ybase = y - 2.0*R

				l = 1
				while l <= chain:
					if whichside == 2:
						if j == 1:
							m = l
						else:
							m = l

					elif whichside == 1:
						if two != 0:
							m = l-1
							ybase = ybaseb
						elif two == 0:
							m = l
							ybase = ybaset
						
					y = ybase - (m)*2.0*R*fact*factors*dir
					l1 = CD.startCD + k
                                	l2 = CD.strandCD[k:k+1]
					loop_image_data3(r,k,CD,l1,l2,x,y,whichside,verbose)

					l = l + 1
					if k == N:
						#print "ending this"
						#print "j is %d and k is %d" % (j,k)
						break
					k = k + 1
				if k == N:
					break

				#draw cross over section
				n = 1
				x = x + R

				while n <=2:
					if j == 1:
						m = chain
					else:
						m = chain+1

					if whichside == 1:
						if two != 0:				
							m = 0
							ybase = ybaset
						else:
							m = 1
							ybase = ybaseb
					if whichside == 2:
						m = 0
						if two != 0:				
							m = 0
							ybase = ybaseb
						else:
							m = 1
							ybase = ybaset
#						ybase = ybaseb

					y = ybase - (m)*2.0*R*factors*fact*dir
					l1 = CD.startCD + k
       	                       		l2 = CD.strandCD[k:k+1]
					loop_image_data3(r,k,CD,l1,l2,x,y,whichside,verbose)
					n = n + 1
					if k == N:
						break
					k = k +1
				
					if CD.numCD == 99:
						x = x + 2.0*R
						y = y + 2.0*R 
				if k == N:
					break

				j = j + 1
				if CD.numCD == 99:
					x = x - R

		j = 1
		if custom == 1 and CD.numCD == 99:
			print "99 section"
			print "cusidnum is %d" %cusidnum
			ij = 0
			max_patch = 0
			ystart = y
                        while ij < r.cuselements[cusidnum]:
                        	if cusidnum == 0:
                                	idoff = 0
                                else:
                                        idoff = 3*(cusidnum - 1)
                                patch_size = r.cusend[ij + idoff] - r.cusstart[ij + idoff] + 1
                                if patch_size > max_patch:
                                        max_patch = patch_size
                                print "ij is %d and patch size is %d"  %(ij,patch_size)
                                ij = ij + 1

			#determine the variables
			print "---there are %d elements" %ij			
			if ij == 2:
				print "this needs work 2 elements"
				leg3x = r.cusstart[3*cusidnum+ 1] - r.cusend[3*cusidnum] -1 
				leg3xup = (leg3x - 1)/2
				leg3xodd = leg3x%2
				leg3xupover = (leg3x - 1)%2
                        	print "max patch is %d leg3x is %d leg3xup is %d and leg3xupover is %d" %(max_patch,leg3x,leg3xup,leg3xupover)

	                        leg2 = r.cusstart[3*cusidnum] - CD.startCD -9 
	                        leg2up = (leg2-1)/2
	                        leg2upover = (leg2-1)%2
		                print "max patch is %d leg2 is %d leg2up is %d and leg2upover is %d" %(max_patch,leg2,leg2up,leg2upover)
				#draw the determined loops and caps
				i = 1
				j = 1
				while i <= 2:
					while j <= leg2up:
                        			y = ystart -2.0*j*R*fact*dir
						loop_image_data(r,k,CD,x,y,whichside,verbose)
                                		j = j + 1
                                		k = k + 1
	                        	offs = .578*R
                                	#draw the cross section
                                	if leg2upover > i - 1:
                                		nums = 2
                                	else:
                                        	nums = 1
 
                               		if nums == 1:
                                        	offs =  1.578*R
                                	n = 0
                                	y = y -2.0*R*fact*dir
                                	while n < nums: 
                                		l1 = CD.startCD +k
                                        	l2 = CD.strandCD[k:k+1]
                                        	l3 = x + offs + n*2.0*R
						loop_image_data3(r,k,CD,l1,l2,l3,y,whichside,verbose)
                                        	k = k + 1
                                        	n = n + 1

                                	x = x + 3.15*R
                                	ystart = y
                                	dir = -1.0*dir
                                	i = i + 1
	                        	j = 1

				#draw down the part to the start of patch
				j = 1
				while j <= 6:
                        		y = ystart -2.0*j*R*fact*dir
					loop_image_data(r,k,CD,x,y,whichside,verbose)
                               		j = j + 1
                               		k = k + 1
                        	#draw patch area
                        	toppatch = y
                        	leftpatch = x
                        	numelem = r.cuselements[cusidnum]
                        	makefeaturebox(r,whichside,leftpatch,toppatch,max_patch,numelem,verbose)

                        	patcha = r.cusend[3*cusidnum] -r.cusstart[3*cusidnum]+ 1
                        	if patcha == max_patch:
                        		buff = 0.0
                        	else:
                                	buff = float(max_patch - patcha)/float(patcha + 1)                                        
                        
				ystart = y
				j = 1
				while j <= patcha:
                                	y = ystart -2.0*j*R*fact*dir*(1.0+buff) 
                                	loop_image_data(r,k,CD,x,y,whichside,verbose)
                                	j = j + 1
                                	k = k + 1

				print " k is %d" %k
                        	botpatch = y
				#draw a tight loop or a long pointing one
				checklong0 = leg2up - (max_patch+6)
				checklong = leg3x - 2*(checklong0+2) 
				xref = x
				xref2 = x + 3.15*R
				if checklong >= 20:
					#print "checklong0 is %d and checklong is %d" %(checklong0,checklong)
					#draw down to the branch point
					j = 1
					ystart = y
                       			while j <= checklong0+2:
                               			y = ystart -2.0*j*R*fact*dir
                               			loop_image_data(r,k,CD,x,y,whichside,verbose)
                               			j = j + 1
                               			k = k + 1

					#determine the size of the 4 areas
					branch = (checklong -2)/4
					branchover = (checklong - 2)%4
					#print "branchover is %d" %branchover
					yover = y-2.0*R*fact*dir
                                	n = 0
       	                        	while n < branch:
               	                        	l1 = CD.startCD +k
                       	                	l2 = CD.strandCD[k:k+1]
                               	        	l3 = x - n*2.0*R
                                       		loop_image_data3(r,k,CD,l1,l2,l3,yover,whichside,verbose)
                                       		n = n + 1
                                       		k = k + 1

                               		offs = .578*R
                               		#draw the cross down section
					xstart = l3
                               		if branchover > 1:
                                       		nums = 2
					elif leg3xodd > 0:
						nums = 2
                               		else:
                                       		nums = 1
 
                               		if nums == 1:
                                       		offs = 1.578*R

                            		#print "nums is %d" %nums
			 		n = 0
					l3 = xstart - 2.0*R
                               		while n < nums: 
	                               		y = yover -offs*fact*dir -n*2.0*R*fact*dir
                                       		l1 = CD.startCD +k
                                       		l2 = CD.strandCD[k:k+1]
                                       		loop_image_data3(r,k,CD,l1,l2,l3,y,whichside,verbose)
                                       		k = k + 1
                                       		n = n + 1

					#branch back to middle
	                        	n = 0
					y = yover -3.15*R*fact*dir
        	                	while n < branch:
                	                	l1 = CD.startCD +k
                        	        	l2 = CD.strandCD[k:k+1]
                                		l3 = xstart + n*2.0*R
                                        	loop_image_data3(r,k,CD,l1,l2,l3,y,whichside,verbose)
                                        	n = n + 1
                                        	k = k + 1

					#second part of the branch
					x = xref2
					n = 0
        	                	while n < branch:
                	        		l1 = CD.startCD +k
                        	        	l2 = CD.strandCD[k:k+1]
                                		l3 = x + n*2.0*R
                                        	loop_image_data3(r,k,CD,l1,l2,l3,y,whichside,verbose)
                                        	n = n + 1
                                        	k = k + 1

                                	offs = .578*R
                                	#draw the cross up section
					xstart = l3
                                	if branchover > 2 or branchover == 2:
                                        	nums = 2
                                	else:
                                        	nums = 1
 
                                	if nums == 1:
                                        	offs = 1.578*R

                               		#print "nums is %d" %nums
					n = 0
					l3 = xstart + 2.0*R
					ystart = y
                                	while n < nums: 
	                                	y = ystart +offs*fact*dir +n*2.0*R*fact*dir
                                        	l1 = CD.startCD +k
                                        	l2 = CD.strandCD[k:k+1]
                                        	loop_image_data3(r,k,CD,l1,l2,l3,y,whichside,verbose)
                                        	k = k + 1
                                        	n = n + 1

					x = l3
					y = yover
	                        	n = 0
        	                	while n < branch:
                	        		l1 = CD.startCD +k
                        	        	l2 = CD.strandCD[k:k+1]
                                		l3 = x - (n+1)*2.0*R
                                        	loop_image_data3(r,k,CD,l1,l2,l3,y,whichside,verbose)
                                        	n = n + 1
                                        	k = k + 1

					j = 1
					dir = -1.0*dir
					x = xref2
					ystart = yover
                        		while j <= checklong0+2:
                                		y = ystart -2.0*j*R*fact*dir
                                		loop_image_data(r,k,CD,x,y,whichside,verbose)
                                		j = j + 1
                                		k = k + 1

				else:
					ystart = y
                        		#draw half of the loop to the next patch element
					j = 1
					while j <= leg3xup:
                        			y = ystart -2.0*j*R*fact*dir
						loop_image_data(r,k,CD,x,y,whichside,verbose)
                               			j = j + 1
                               			k = k + 1

					print " k is %d" %k
	                		offs = .578*R
                        		#draw the cross section
                        		if leg3xupover > 0:
                                		nums = 2
                        		else:
                                		nums = 1
 
                        		if nums == 1:
                                		offs = 1.578*R

                        		n = 0
                        		y = y -2.0*R*fact*dir
                        		while n < nums: 
                                		l1 = CD.startCD +k
                                		l2 = CD.strandCD[k:k+1]
                                		l3 = x + offs + n*2.0*R
						loop_image_data3(r,k,CD,l1,l2,l3,y,whichside,verbose)
                                		k = k + 1
                                		n = n + 1

                        		x = x + 3.15*R
                        		ystart = y
                        		dir = -1.0*dir
					print " j is %d" %j
                        		j = 1
					while j <= leg3xup:
                        			y = ystart -2.0*j*R*fact*dir
						loop_image_data(r,k,CD,x,y,whichside,verbose)
                               			j = j + 1
                               			k = k + 1

					print " k is %d" %k

				#put in second patch
                        	patchb = r.cusend[3*cusidnum + 1] -r.cusstart[3*cusidnum+1]+ 1
                        	if patchb == max_patch:
                        		buff = 0.0
                        	else:
                                	buff = float(max_patch - patchb)/float(patchb + 1)                                        

				print "patchb is %d" %patchb                        
				ystart = y
				j = 1
				while j <= patchb:
                                	y = ystart -2.0*j*R*fact*dir*(1.0+buff) 
                                	loop_image_data(r,k,CD,x,y,whichside,verbose)
                                	j = j + 1
                                	k = k + 1

				print " k is %d" %k
				print "spacer is 6" 
				ystart = y
				j = 1
				while j <= 6:
                                	y = ystart -2.0*j*R*fact*dir*(1.0+buff) 
                                	loop_image_data(r,k,CD,x,y,whichside,verbose)
                                	j = j + 1
                                	k = k + 1

				print " k is %d" %k
	                	offs = .578*R
                        	#draw the cross section
                        	nums = 2
                        	n = 0
                        	y = y -2.0*R*fact*dir
                        	while n < nums: 
                                	l1 = CD.startCD +k
                                	l2 = CD.strandCD[k:k+1]
                                	l3 = x + offs + n*2.0*R
					loop_image_data3(r,k,CD,l1,l2,l3,y,whichside,verbose)
                                	k = k + 1
                                	n = n + 1

                        	x = x + 3.15*R
                        	dir = -1.0*dir

				#determine the rest of the sequence
                        	y = y -2.0*R*fact*dir

				resleft = CD.clenCD -k
				lop = resleft/(chain + 1)
				lox = resleft%(chain+1.0)
				if lox > 0:
					lop = lop+1
				j = 1
				ystart = y
				while j <= lop:
					print "y is %f in pass %d" %(y,j)

					l = 1
					while l <= chain:
						y = ystart - (l-1)*2.0*R*dir*fact
						if CD.numCD == 0:
							l1 = CD.endCD - k
	        	                		l2 = rev[k:k+1]
							if l1 != 0:
								loop_image_data2(r,CD,l1,l2,x,y,whichside,verbose)

						else:
							l1 = CD.startCD + k
                                			l2 = CD.strandCD[k:k+1]
							loop_image_data3(r,k,CD,l1,l2,x,y,whichside,verbose)

						l = l + 1
						if k == N:
							#print "ending this"
							#print "j is %d and k is %d" % (j,k)
							break
						k = k + 1
					if k == N:
						break

					#draw cross over section
					n = 0
					nums = 2


					y = y -2.0*R*dir*fact
					offs = .578*R
					while n < nums:
						if CD.numCD == 99:
							x2 = x + offs + n*2.0*R
						else:
							x = x - offs - n*2.0*R
						if CD.numCD == 0:
							l1 = CD.endCD - k
	        	                		l2 = rev[k:k+1]
							if l1 != 0:
								loop_image_data2(r,CD,l1,l2,x,y,whichside,verbose)
						else:
							l1 = CD.startCD + k
       	                        			l2 = CD.strandCD[k:k+1]
							loop_image_data3(r,k,CD,l1,l2,x2,y,whichside,verbose)
						n = n + 1
						if k == N:
							break
						k = k +1
				
					dir = -1.0*dir
					if CD.numCD == 99:
						x = x + 3.15*R
						y = y - 2.0*R*dir*fact 
					else:
						x = x - 3.15*R
						y = y + 2.0*R*dir*fact
 					if k == N:
						break

					j = j + 1

					ystart = y
			if ij == 3:
				leg4 = r.cusstart[3*cusidnum +2] - r.cusend[3*cusidnum +1] -1
				leg4up = (leg4 - 1)/2
				leg4upover = (leg4 - 1)%2
                        	print "max patch is %d leg4 is %d leg4up is %d and leg4upover is %d" %(max_patch,leg4,leg4up,leg4upover)

				leg3x = r.cusstart[3*cusidnum+ 1] - r.cusend[3*cusidnum] -1 
				leg3xup = (leg3x - 1)/2
				leg3xodd = leg3x%2
				leg3xupover = (leg3x - 1)%2
                        	print "max patch is %d leg3x is %d leg3xup is %d and leg3xupover is %d" %(max_patch,leg3x,leg3xup,leg3xupover)

	                        leg2 = r.cusstart[3*cusidnum] - CD.startCD -(3+leg4up)
	                        leg2up = (leg2-1)/2
	                        leg2upover = (leg2-1)%2
		                print "max patch is %d leg2 is %d leg2up is %d and leg2upover is %d" %(max_patch,leg2,leg2up,leg2upover)
				#draw the determined loops and caps
				i = 1
				j = 1
				while i <= 2:
					while j <= leg2up:
                        			y = ystart -2.0*j*R*fact*dir
						loop_image_data(r,k,CD,x,y,whichside,verbose)
                                		j = j + 1
                                		k = k + 1
	                        	offs = .578*R
                                	#draw the cross section
                                	if leg2upover > i - 1:
                                		nums = 2
                                	else:
                                        	nums = 1
 
                               		if nums == 1:
                                        	offs =  1.578*R
                                	n = 0
                                	y = y -2.0*R*fact*dir
                                	while n < nums: 
                                		l1 = CD.startCD +k
                                        	l2 = CD.strandCD[k:k+1]
                                        	l3 = x + offs + n*2.0*R
						loop_image_data3(r,k,CD,l1,l2,l3,y,whichside,verbose)
                                        	k = k + 1
                                        	n = n + 1

                                	x = x + 3.15*R
                                	ystart = y
                                	dir = -1.0*dir
                                	i = i + 1
	                        	j = 1

				#draw down the part to the start of patch
				j = 1
				while j <= leg4up:
                        		y = ystart -2.0*j*R*fact*dir
					loop_image_data(r,k,CD,x,y,whichside,verbose)
                               		j = j + 1
                               		k = k + 1
                        	#draw patch area
                        	toppatch = y
                        	leftpatch = x
                        	numelem = r.cuselements[cusidnum]
                        	makefeaturebox(r,whichside,leftpatch,toppatch,max_patch,numelem,verbose)

                        	patcha = r.cusend[3*cusidnum] -r.cusstart[3*cusidnum]+ 1
                        	if patcha == max_patch:
                        		buff = 0.0
                        	else:
                                	buff = float(max_patch - patcha)/float(patcha + 1)                                        
                        
				ystart = y
				j = 1
				while j <= patcha:
                                	y = ystart -2.0*j*R*fact*dir*(1.0+buff) 
                                	loop_image_data(r,k,CD,x,y,whichside,verbose)
                                	j = j + 1
                                	k = k + 1

				#print " k is %d" %k
                        	botpatch = y
				#draw a tight loop or a long pointing one
				leg2check = leg2 - 2*leg2up
				if leg2check == 2:
					leg2up = leg2up+1
				checklong0 = leg2up - (max_patch+leg4up)
				checklong = leg3x - 2*(checklong0+2) 
				checklongodd = checklong%2
				print "checklongodd is %d" %checklongodd
				xref = x
				xref2 = x + 3.15*R
				if checklongodd == 1:
					print "working with odd data";

				if checklong >= 20:
					#print "checklong0 is %d and checklong is %d" %(checklong0,checklong)
					#draw down to the branch point
					j = 1
					ystart = y
					spacer = 2
					#determine the size of the 4 areas
					branch = (checklong -2)/4
					branchover = (checklong - 2)%4
					#print "branch is %d and branchover is %d" %(branch,branchover)
					total = 2*(checklong0+spacer)+4*branch+branchover
					diff = leg3x - total
					print "total is %d" %total
					if total < leg3x and diff == 2:
						spacer = 3
						if branchover == 0:
							spacer = 2
						if branchover == 2:
							spacer = 2
						if checklongodd == 1:
							leg3xxodd = leg3x%4
							if leg3xxodd == 3:
								spacer = 4
								branch = branch -1
								branchover = branchover +2
								print "adjusted branch and branchover"
                       			while j <= checklong0+spacer:
                               			y = ystart -2.0*j*R*fact*dir
                               			loop_image_data(r,k,CD,x,y,whichside,verbose)
                               			j = j + 1
                               			k = k + 1

					yover = y-2.0*R*dir*fact
                                	n = 0
       	                        	while n < branch:
               	                        	l1 = CD.startCD +k
                       	                	l2 = CD.strandCD[k:k+1]
                               	        	l3 = x - n*2.0*R
                                       		loop_image_data3(r,k,CD,l1,l2,l3,yover,whichside,verbose)
                                       		n = n + 1
                                       		k = k + 1

                               		offs = .578*R
                               		#draw the cross down section
					xstart = l3
                               		if branchover > 1:
                                       		nums = 2
					elif leg3xodd > 0:
						nums = 2
                               		else:
                                       		nums = 1
 
                               		if nums == 1:
                                       		offs = 1.578*R

                            		#print "nums is %d" %nums
			 		n = 0
					l3 = xstart - 2.0*R
                               		while n < nums: 
	                               		y = yover -offs*dir*fact -n*2.0*R*dir*fact
                                       		l1 = CD.startCD +k
                                       		l2 = CD.strandCD[k:k+1]
                                       		loop_image_data3(r,k,CD,l1,l2,l3,y,whichside,verbose)
                                       		k = k + 1
                                       		n = n + 1

					#branch back to middle
	                        	n = 0
					y = yover -3.15*R*dir*fact
        	                	while n < branch:
                	                	l1 = CD.startCD +k
                        	        	l2 = CD.strandCD[k:k+1]
                                		l3 = xstart + n*2.0*R
                                        	loop_image_data3(r,k,CD,l1,l2,l3,y,whichside,verbose)
                                        	n = n + 1
                                        	k = k + 1

					#second part of the branch
					x = xref2
					n = 0
        	                	while n < branch:
                	        		l1 = CD.startCD +k
                        	        	l2 = CD.strandCD[k:k+1]
                                		l3 = x + n*2.0*R
                                        	loop_image_data3(r,k,CD,l1,l2,l3,y,whichside,verbose)
                                        	n = n + 1
                                        	k = k + 1

                                	offs = .578*R
                                	#draw the cross up section
					xstart = l3
					if branchover == 3:
						nums = 1
                                	elif branchover == 2:
                                        	nums = 2
                                	else:
                                        	nums = 1
 
                                	if nums == 1:
                                        	offs = 1.578*R

                               		#print "nums is %d" %nums
					n = 0
					l3 = xstart + 2.0*R
					ystart = y
                                	while n < nums: 
	                                	y = ystart +offs*dir*fact +n*2.0*R*dir*fact
                                        	l1 = CD.startCD +k
                                        	l2 = CD.strandCD[k:k+1]
                                        	loop_image_data3(r,k,CD,l1,l2,l3,y,whichside,verbose)
                                        	k = k + 1
                                        	n = n + 1

					x = l3
					y = yover
	                        	n = 0
        	                	while n < branch:
                	        		l1 = CD.startCD +k
                        	        	l2 = CD.strandCD[k:k+1]
                                		l3 = x - (n+1)*2.0*R
                                        	loop_image_data3(r,k,CD,l1,l2,l3,y,whichside,verbose)
                                        	n = n + 1
                                        	k = k + 1

					j = 1
					dir = -1.0*dir
					x = xref2
					ystart = yover
                        		while j <= checklong0+spacer:
                                		y = ystart -2.0*j*R*fact*dir
                                		loop_image_data(r,k,CD,x,y,whichside,verbose)
                                		j = j + 1
                                		k = k + 1

				else:
					ystart = y
                        		#draw half of the loop to the next patch element
					j = 1
					while j <= leg3xup:
                        			y = ystart -2.0*j*R*fact*dir
						loop_image_data(r,k,CD,x,y,whichside,verbose)
                               			j = j + 1
                               			k = k + 1

					print " k is %d" %k
	                		offs = .578*R
                        		#draw the cross section
                        		if leg3xupover > 0:
                                		nums = 2
                        		else:
                                		nums = 1
 
                        		if nums == 1:
                                		offs = 1.578*R

                        		n = 0
                        		y = y -2.0*R*fact*dir
                        		while n < nums: 
                                		l1 = CD.startCD +k
                                		l2 = CD.strandCD[k:k+1]
                                		l3 = x + offs + n*2.0*R
						loop_image_data3(r,k,CD,l1,l2,l3,y,whichside,verbose)
                                		k = k + 1
                                		n = n + 1

                        		x = x + 3.15*R
                        		ystart = y
                        		dir = -1.0*dir
					print " j is %d" %j
                        		j = 1
					while j <= leg3xup:
                        			y = ystart -2.0*j*R*fact*dir
						loop_image_data(r,k,CD,x,y,whichside,verbose)
                               			j = j + 1
                               			k = k + 1

					print " k is %d" %k

				#put in second patch
                        	patchb = r.cusend[3*cusidnum + 1] -r.cusstart[3*cusidnum+1]+ 1
                        	if patchb == max_patch:
                        		buff = 0.0
                        	else:
                                	buff = float(max_patch - patchb)/float(patchb + 1)                                        

				print "patchb is %d" %patchb                        
				ystart = y
				j = 1
				while j <= patchb:
                                	y = ystart -2.0*j*R*fact*dir*(1.0+buff) 
                                	loop_image_data(r,k,CD,x,y,whichside,verbose)
                                	j = j + 1
                                	k = k + 1

				print " k is %d" %k

				#draw the loop between parts b and c
				ystart = y
                        	j = 1
				print "leg4up is %d" %leg4up
				while j <= leg4up:
					#print "working on pass %d x is %f and y is %f" %(j,x,y)
                        		y = ystart -2.0*j*R*fact*dir
					loop_image_data(r,k,CD,x,y,whichside,verbose)
                               		j = j + 1
                               		k = k + 1

				print " k is %d" %k

	                	offs = .578*R
                        	#draw the cross section
	                        if leg4upover > 0:
        	                        nums = 2
                	        else:
                        	        nums = 1
 
                        	if nums == 1:
                                	offs = 1.578*R

                       		n = 0
                        	y = y -2.0*R*fact*dir
                        	while n < nums: 
                                	l1 = CD.startCD +k
                                	l2 = CD.strandCD[k:k+1]
                                	l3 = x + offs + n*2.0*R
					loop_image_data3(r,k,CD,l1,l2,l3,y,whichside,verbose)
                                	k = k + 1
                                	n = n + 1

                        	x = x + 3.15*R
                        	ystart = y
                        	dir = -1.0*dir
				print " j is %d" %j
                        	j = 1
				while j <= leg4up:
                        		y = ystart -2.0*j*R*fact*dir
					loop_image_data(r,k,CD,x,y,whichside,verbose)
                               		j = j + 1
                               		k = k + 1

				print " k is %d" %k
				#put in patchc
                        	patchc = r.cusend[3*cusidnum + 2] -r.cusstart[3*cusidnum+2]+ 1
                        	if patchc == max_patch:
                        		buff = 0.0
                        	else:
                                	buff = float(max_patch - patchc)/float(patchc + 1)                                        

				#print "patchc is %d and buff is %f" %(patchc,buff)                        
				ystart = y
				j = 1
				while j <= patchc:
					print "k is %d" %k
                                	y = ystart -2.0*j*fact*dir*R*(1.0+buff)
                                	loop_image_data(r,k,CD,x,y,whichside,verbose)
                                	j = j + 1
                                	k = k + 1

				print " k is %d" %k

				#down to get to a good point to start repeating
				leg5 = 30 - (leg4up + max_patch)
				ystart = y -2.0*R*fact*dir*buff
				j = 1
				while j <= leg5:
                                	y = ystart -2.0*j*R*fact*dir
                                	loop_image_data(r,k,CD,x,y,whichside,verbose)
                                	j = j + 1
                                	k = k + 1

				print " k is %d" %k
	                	offs = .578*R
                        	#draw the cross section
                        	nums = 2
                        	n = 0
                        	y = y -2.0*R*fact*dir
                        	while n < nums: 
                                	l1 = CD.startCD +k
                                	l2 = CD.strandCD[k:k+1]
                                	l3 = x + offs + n*2.0*R
					loop_image_data3(r,k,CD,l1,l2,l3,y,whichside,verbose)
                                	k = k + 1
                                	n = n + 1

                        	x = x + 3.15*R
                        	dir = -1.0*dir
				print " j is %d" %j

				#determine the rest of the sequence
                        	y = y -2.0*R*fact*dir

				resleft = CD.clenCD -k
				lop = resleft/(chain + 1)
				lox = resleft%(chain+1.0)
				if lox > 0:
					lop = lop+1
				j = 1
				ystart = y
				dir = -1.0*dir
				while j <= lop:
					print "y is %f in pass %d" %(y,j)

					l = 1
					while l <= chain:
						y = ystart + (l-1)*2.0*R*dir*fact
						if CD.numCD == 0:
							l1 = CD.endCD - k
	        	                		l2 = rev[k:k+1]
							if l1 != 0:
								loop_image_data2(r,CD,l1,l2,x,y,whichside,verbose)

						else:
							l1 = CD.startCD + k
                                			l2 = CD.strandCD[k:k+1]
							loop_image_data3(r,k,CD,l1,l2,x,y,whichside,verbose)

						l = l + 1
						if k == N:
							#print "ending this"
							#print "j is %d and k is %d" % (j,k)
							break
						k = k + 1
					if k == N:
						break

					#draw cross over section
					n = 0
					nums = 2


					y = y +2.0*R*dir*fact
					offs = .578*R
					while n < nums:
						if CD.numCD == 99:
							x2 = x + offs + n*2.0*R
						else:
							x = x - offs - n*2.0*R
						if CD.numCD == 0:
							l1 = CD.endCD - k
	        	                		l2 = rev[k:k+1]
							if l1 != 0:
								loop_image_data2(r,CD,l1,l2,x,y,whichside,verbose)
						else:
							l1 = CD.startCD + k
       	                        			l2 = CD.strandCD[k:k+1]
							loop_image_data3(r,k,CD,l1,l2,x2,y,whichside,verbose)
						n = n + 1
						if k == N:
							break
						k = k +1
				
					dir = -1.0*dir
					if CD.numCD == 99:
						x = x + 3.15*R
						y = y + 2.0*R*dir*fact 
					else:
						x = x - 3.15*R
						y = y + 2.0*R*dir*fact
 					if k == N:
						break

					j = j + 1

					ystart = y
     			if ij == 4:
				print "needs work 4 elements"

				leg5 = r.cusstart[3*cusidnum +3] - r.cusend[3*cusidnum +2] -1
				leg5up = (leg5 - 1)/2
				leg5upover = (leg5 - 1)%2
                        	print "max patch is %d leg5 is %d leg5up is %d and leg5upover is %d" %(max_patch,leg5,leg5up,leg5upover)

				leg4 = r.cusstart[3*cusidnum +2] - r.cusend[3*cusidnum +1] -1
				leg4up = (leg4 - 1)/2
				leg4upover = (leg4 - 1)%2
                        	print "max patch is %d leg4 is %d leg4up is %d and leg4upover is %d" %(max_patch,leg4,leg4up,leg4upover)

				leg3x = r.cusstart[3*cusidnum+ 1] - r.cusend[3*cusidnum] -1 
				leg3xup = (leg3x - 1)/2
				leg3xodd = leg3x%2
				leg3xupover = (leg3x - 1)%2
                        	print "max patch is %d leg3x is %d leg3xup is %d and leg3xupover is %d" %(max_patch,leg3x,leg3xup,leg3xupover)

	                        leg2 = r.cusstart[3*cusidnum] - CD.startCD -(3+leg4up)
	                        leg2up = (leg2-1)/2
	                        leg2upover = (leg2-1)%2
		                print "max patch is %d leg2 is %d leg2up is %d and leg2upover is %d" %(max_patch,leg2,leg2up,leg2upover)
				#draw the determined loops and caps
				i = 1
				j = 1
				while i <= 2:
					while j <= leg2up:
                        			y = ystart -2.0*j*R*fact*dir
						loop_image_data(r,k,CD,x,y,whichside,verbose)
                                		j = j + 1
                                		k = k + 1
	                        	offs = .578*R
                                	#draw the cross section
                                	if leg2upover > i - 1:
                                		nums = 2
                                	else:
                                        	nums = 1
 
                               		if nums == 1:
                                        	offs =  1.578*R
                                	n = 0
                                	y = y -2.0*R*fact*dir
                                	while n < nums: 
                                		l1 = CD.startCD +k
                                        	l2 = CD.strandCD[k:k+1]
                                        	l3 = x + offs + n*2.0*R
						loop_image_data3(r,k,CD,l1,l2,l3,y,whichside,verbose)
                                        	k = k + 1
                                        	n = n + 1

                                	x = x + 3.15*R
                                	ystart = y
                                	dir = -1.0*dir
                                	i = i + 1
	                        	j = 1

				#draw down the part to the start of patch
				j = 1
				while j <= leg4up:
                        		y = ystart -2.0*j*R*fact*dir
					loop_image_data(r,k,CD,x,y,whichside,verbose)
                               		j = j + 1
                               		k = k + 1
                        	#draw patch area
                        	toppatch = y
                        	leftpatch = x
                        	numelem = r.cuselements[cusidnum]
                        	makefeaturebox(r,whichside,leftpatch,toppatch,max_patch,numelem,verbose)

                        	patcha = r.cusend[3*cusidnum] -r.cusstart[3*cusidnum]+ 1
                        	if patcha == max_patch:
                        		buff = 0.0
                        	else:
                                	buff = float(max_patch - patcha)/float(patcha + 1)                                        
                        
				ystart = y
				j = 1
				while j <= patcha:
                                	y = ystart -2.0*j*R*fact*dir*(1.0+buff) 
                                	loop_image_data(r,k,CD,x,y,whichside,verbose)
                                	j = j + 1
                                	k = k + 1

				print " k is %d" %k
                        	botpatch = y
				spacer2 = 2
				#draw a tight loop or a long pointing one
				checklong0 = leg2up - (max_patch+leg4up)
				checklong = leg3x - 2*(checklong0+spacer2) 
				xref = x
				xref2 = x + 3.15*R
				if checklong >= 20:
					#print "checklong0 is %d and checklong is %d" %(checklong0,checklong)

					#determine the size of the 4 areas
					branch = (checklong -2)/4
					branchover = (checklong - 2)%4
					#print "branch is %d and branchover is %d" %(branch,branchover)
					total2 = 2*(checklong0 + spacer2) + 4*branch + branchover
					diff = leg3x - total2
					if total2 < leg3x and diff == 2:
						spacer2 = 3
					#draw down to the branch point
					j = 1
					ystart = y
                       			while j <= checklong0+spacer2:
                               			y = ystart -2.0*j*R*fact*dir
                               			loop_image_data(r,k,CD,x,y,whichside,verbose)
                               			j = j + 1
                               			k = k + 1

					yover = y-2.0*R*dir*fact
                                	n = 0
       	                        	while n < branch:
               	                        	l1 = CD.startCD +k
                       	                	l2 = CD.strandCD[k:k+1]
                               	        	l3 = x - n*2.0*R
                                       		loop_image_data3(r,k,CD,l1,l2,l3,yover,whichside,verbose)
                                       		n = n + 1
                                       		k = k + 1

                               		offs = .578*R
                               		#draw the cross down section
					xstart = l3
                               		if branchover > 1:
                                       		nums = 2
					elif leg3xodd > 0:
						nums = 2
                               		else:
                                       		nums = 1
 
                               		if nums == 1:
                                       		offs = 1.578*R

                            		#print "nums is %d" %nums
			 		n = 0
					l3 = xstart - 2.0*R
                               		while n < nums: 
	                               		y = yover -offs*dir*fact -n*2.0*R*dir*fact
                                       		l1 = CD.startCD +k
                                       		l2 = CD.strandCD[k:k+1]
                                       		loop_image_data3(r,k,CD,l1,l2,l3,y,whichside,verbose)
                                       		k = k + 1
                                       		n = n + 1

					#branch back to middle
	                        	n = 0
					y = yover -3.15*R*dir*fact
        	                	while n < branch:
                	                	l1 = CD.startCD +k
                        	        	l2 = CD.strandCD[k:k+1]
                                		l3 = xstart + n*2.0*R
                                        	loop_image_data3(r,k,CD,l1,l2,l3,y,whichside,verbose)
                                        	n = n + 1
                                        	k = k + 1

					#second part of the branch
					x = xref2
					n = 0
        	                	while n < branch:
                	        		l1 = CD.startCD +k
                        	        	l2 = CD.strandCD[k:k+1]
                                		l3 = x + n*2.0*R
                                        	loop_image_data3(r,k,CD,l1,l2,l3,y,whichside,verbose)
                                        	n = n + 1
                                        	k = k + 1

                                	offs = .578*R
                                	#draw the cross up section
					xstart = l3
                                	if branchover == 3:
						nums = 1
					elif branchover == 2:
                                        	nums = 2
                                	else:
                                        	nums = 1
 
                                	if nums == 1:
                                        	offs = 1.578*R

                               		#print "nums is %d" %nums
					n = 0
					l3 = xstart + 2.0*R
					ystart = y
                                	while n < nums: 
	                                	y = ystart +offs*dir*fact +n*2.0*R*dir*fact
                                        	l1 = CD.startCD +k
                                        	l2 = CD.strandCD[k:k+1]
                                        	loop_image_data3(r,k,CD,l1,l2,l3,y,whichside,verbose)
                                        	k = k + 1
                                        	n = n + 1

					x = l3
					y = yover
	                        	n = 0
        	                	while n < branch:
                	        		l1 = CD.startCD +k
                        	        	l2 = CD.strandCD[k:k+1]
                                		l3 = x - (n+1)*2.0*R
                                        	loop_image_data3(r,k,CD,l1,l2,l3,y,whichside,verbose)
                                        	n = n + 1
                                        	k = k + 1

					j = 1
					dir = -1.0*dir
					x = xref2
					ystart = yover
                        		while j <= checklong0+spacer2:
                                		y = ystart -2.0*j*R*fact*dir
                                		loop_image_data(r,k,CD,x,y,whichside,verbose)
                                		j = j + 1
                                		k = k + 1

				else:
					ystart = y
                        		#draw half of the loop to the next patch element
					j = 1
					while j <= leg3xup:
                        			y = ystart -2.0*j*R*fact*dir
						loop_image_data(r,k,CD,x,y,whichside,verbose)
                               			j = j + 1
                               			k = k + 1

					print " k is %d" %k
	                		offs = .578*R
                        		#draw the cross section
                        		if leg3xupover > 0:
                                		nums = 2
                        		else:
                                		nums = 1
 
                        		if nums == 1:
                                		offs = 1.578*R

                        		n = 0
                        		y = y -2.0*R*fact*dir
                        		while n < nums: 
                                		l1 = CD.startCD +k
                                		l2 = CD.strandCD[k:k+1]
                                		l3 = x + offs + n*2.0*R
						loop_image_data3(r,k,CD,l1,l2,l3,y,whichside,verbose)
                                		k = k + 1
                                		n = n + 1

                        		x = x + 3.15*R
                        		ystart = y
                        		dir = -1.0*dir
					print " j is %d" %j
                        		j = 1
					while j <= leg3xup:
                        			y = ystart -2.0*j*R*fact*dir
						loop_image_data(r,k,CD,x,y,whichside,verbose)
                               			j = j + 1
                               			k = k + 1

					print " k is %d" %k

				#put in second patch
                        	patchb = r.cusend[3*cusidnum + 1] -r.cusstart[3*cusidnum+1]+ 1
                        	if patchb == max_patch:
                        		buff = 0.0
                        	else:
                                	buff = float(max_patch - patchb)/float(patchb + 1)                                        

				print "patchb is %d" %patchb                        
				ystart = y
				j = 1
				while j <= patchb:
                                	y = ystart -2.0*j*R*fact*dir*(1.0+buff) 
                                	loop_image_data(r,k,CD,x,y,whichside,verbose)
                                	j = j + 1
                                	k = k + 1

				print " k is %d" %k

				#draw the loop between parts b and c
				ystart = y
                        	j = 1
				while j <= leg4up:
                        		y = ystart -2.0*j*R*fact*dir
					loop_image_data(r,k,CD,x,y,whichside,verbose)
                               		j = j + 1
                               		k = k + 1

				print " k is %d" %k

	                	offs = .578*R
                        	#draw the cross section
	                        if leg4upover > 0:
        	                        nums = 2
                	        else:
                        	        nums = 1
 
                        	if nums == 1:
                                	offs = 1.578*R

                       		n = 0
                        	y = y -2.0*R*fact*dir
                        	while n < nums: 
                                	l1 = CD.startCD +k
                                	l2 = CD.strandCD[k:k+1]
                                	l3 = x + offs + n*2.0*R
					loop_image_data3(r,k,CD,l1,l2,l3,y,whichside,verbose)
                                	k = k + 1
                                	n = n + 1

                        	x = x + 3.15*R
                        	ystart = y
                        	dir = -1.0*dir
				print " j is %d" %j
                        	j = 1
				while j <= leg4up:
                        		y = ystart -2.0*j*R*fact*dir
					loop_image_data(r,k,CD,x,y,whichside,verbose)
                               		j = j + 1
                               		k = k + 1

				print " k is %d" %k
				#put in patchc
                        	patchc = r.cusend[3*cusidnum + 2] -r.cusstart[3*cusidnum+2]+ 1
                        	if patchc == max_patch:
                        		buff = 0.0
                        	else:
                                	buff = float(max_patch - patchc)/float(patchc + 1)                                        

				#print "patchc is %d and buff is %f" %(patchc,buff)                        
				ystart = y
				j = 1
				while j <= patchc:
					print "k is %d" %k
                                	y = ystart -2.0*j*fact*dir*R*(1.0+buff)
                                	loop_image_data(r,k,CD,x,y,whichside,verbose)
                                	j = j + 1
                                	k = k + 1

				print " k is %d" %k

				print "reached the next patch section"
				ystart = y -2.0*R*fact*dir*buff
                        	j = 1
				while j <= leg5up:
                        		y = ystart -2.0*j*R*fact*dir
					loop_image_data(r,k,CD,x,y,whichside,verbose)
                               		j = j + 1
                               		k = k + 1

				print " k is %d" %k

	                	offs = .578*R
                        	#draw the cross section
	                        if leg5upover > 0:
        	                        nums = 2
                	        else:
                        	        nums = 1
 
                        	if nums == 1:
                                	offs = 1.578*R

                       		n = 0
                        	y = y -2.0*R*fact*dir
                        	while n < nums: 
                                	l1 = CD.startCD +k
                                	l2 = CD.strandCD[k:k+1]
                                	l3 = x + offs + n*2.0*R
					loop_image_data3(r,k,CD,l1,l2,l3,y,whichside,verbose)
                                	k = k + 1
                                	n = n + 1

                        	x = x + 3.15*R
                        	ystart = y
                        	dir = -1.0*dir
				print " j is %d" %j
                        	j = 1
				while j <= leg5up:
                        		y = ystart -2.0*j*R*fact*dir
					loop_image_data(r,k,CD,x,y,whichside,verbose)
                               		j = j + 1
                               		k = k + 1

				print " k is %d" %k

                        	patchd = r.cusend[3*cusidnum + 3] -r.cusstart[3*cusidnum+3]+ 1
                        	if patchd == max_patch:
                        		buff = 0.0
                        	else:
                                	buff = float(max_patch - patchd)/float(patchd + 1)                                        

				#print "patchd is %d and buff is %f" %(patchd,buff)                        
				ystart = y
				j = 1
				while j <= patchd:
					print "k is %d" %k
                                	y = ystart -2.0*j*fact*dir*R*(1.0+buff)
                                	loop_image_data(r,k,CD,x,y,whichside,verbose)
                                	j = j + 1
                                	k = k + 1

				print " k is %d" %k
				#down to get to a good point to start repeating
				leg6 = 30 - (leg4up + max_patch)
				if leg6 > leg4up:
					leg6 = leg4up

				ystart = y -2.0*R*fact*dir*buff
				j = 1
				while j <= leg6:
                                	y = ystart -2.0*j*R*fact*dir
                                	loop_image_data(r,k,CD,x,y,whichside,verbose)
                                	j = j + 1
                                	k = k + 1

				print " k is %d" %k
	                	offs = .578*R
                        	#draw the cross section
                        	nums = 2
                        	n = 0
                        	y = y -2.0*R*fact*dir
                        	while n < nums: 
                                	l1 = CD.startCD +k
                                	l2 = CD.strandCD[k:k+1]
                                	l3 = x + offs + n*2.0*R
					loop_image_data3(r,k,CD,l1,l2,l3,y,whichside,verbose)
                                	k = k + 1
                                	n = n + 1

                        	x = x + 3.15*R
                        	dir = -1.0*dir
				#determine the rest of the sequence
                        	y = y -2.0*R*fact*dir

				resleft = CD.clenCD -k
				if chain < leg2up:
					chain = leg2up

				lop = resleft/(chain + 1)
				lox = resleft%(chain+1.0)
				if lox > 0:
					lop = lop+1
				j = 1
				ystart = y
				dir = -1.0*dir
				while j <= lop:
					print "y is %f in pass %d" %(y,j)

					l = 1
					while l <= chain:
						y = ystart + (l-1)*2.0*R*dir*fact
						if CD.numCD == 0:
							l1 = CD.endCD - k
	        	                		l2 = rev[k:k+1]
							if l1 != 0:
								loop_image_data2(r,CD,l1,l2,x,y,whichside,verbose)

						else:
							l1 = CD.startCD + k
                                			l2 = CD.strandCD[k:k+1]
							loop_image_data3(r,k,CD,l1,l2,x,y,whichside,verbose)

						l = l + 1
						if k == N:
							#print "ending this"
							#print "j is %d and k is %d" % (j,k)
							break
						k = k + 1
					if k == N:
						break

					#draw cross over section
					n = 0
					nums = 2


					y = y +2.0*R*dir*fact
					offs = .578*R
					while n < nums:
						if CD.numCD == 99:
							x2 = x + offs + n*2.0*R
						else:
							x = x - offs - n*2.0*R
						if CD.numCD == 0:
							l1 = CD.endCD - k
	        	                		l2 = rev[k:k+1]
							if l1 != 0:
								loop_image_data2(r,CD,l1,l2,x,y,whichside,verbose)
						else:
							l1 = CD.startCD + k
       	                        			l2 = CD.strandCD[k:k+1]
							loop_image_data3(r,k,CD,l1,l2,x2,y,whichside,verbose)
						n = n + 1
						if k == N:
							break
						k = k +1
				
					dir = -1.0*dir
					if CD.numCD == 99:
						x = x + 3.15*R
						y = y + 2.0*R*dir*fact 
					else:
						x = x - 3.15*R
						y = y + 2.0*R*dir*fact
 					if k == N:
						break

					j = j + 1

					ystart = y

		elif custom == 1 and CD.numCD == 0:
			print "starting to work on this part"
			print "cusidnum is %d" %cusidnum
			ij = 0
			max_patch = 0
			ystart = ybaseb
                        while ij < r.cuselements[cusidnum]:
                        	if cusidnum == 0:
                                	idoff = 0
                                else:
                                        idoff = 3*(cusidnum - 1)
                                patch_size = r.cusend[ij + idoff] - r.cusstart[ij + idoff] + 1
                                if patch_size > max_patch:
                                        max_patch = patch_size
                                print "ij is %d and patch size is %d"  %(ij,patch_size)
                                ij = ij + 1

			#determine the variables - they need to be reversed
			#patch information
			#first patch loop
			leg3x = r.cusstart[3*cusidnum+ 1] - r.cusend[3*cusidnum] -1 
			leg3xup = (leg3x - 1)/2
			leg3xupover = (leg3x - 1)%2
                        print "max patch is %d leg3x is %d leg3xup is %d and leg3xupover is %d" %(max_patch,leg3x,leg3xup,leg3xupover)

			#second patch loop
			leg4 = r.cusstart[3*cusidnum+2] - r.cusend[3*cusidnum +1] - 1
			leg4up = (leg4 - 1)/2
			leg4upover = (leg4 - 1)%2
                        print "max patch is %d leg4 is %d leg4up is %d and leg4upover is %d" %(max_patch,leg4,leg4up,leg4upover)

			#location of last patch from the connection point
                        leg2 = CD.endCD -r.cusend[3*cusidnum+2]
                        cycles = (leg2-1)/(chain+2)
                        cyclesover = (leg2-1)%(chain+2)
                        print "max patch is %d leg2 is %d cycles is %d and cyclesover is %d" %(max_patch,leg2,cycles,cyclesover)

			#check to see that the found area is on a even number leg of the structure
			cycles_check = (cycles+1)%2
			cycles_check2 = chain - (cyclesover+max_patch+leg4up)
			
			if cycles_check == 0 and cycles_check2 >= 0:
				print "chain length 30 works"
			else:
				i = 1
				while i <=10:
					cyclesx = (leg2-1)/(chain+i+2)
					cyclesxover = (leg2-1)%(chain+i+2)
					cyclesx_check = (cyclesx+1)%2
					cyclesx_check2 = chain+i - (cyclesxover+max_patch+leg4up)
					if cyclesx_check == 0 and  cyclesx_check2 >= 0:
						chain = chain +i
						cycles = cyclesx
						cyclesover = cyclesxover
						print "chain length %d works with a %d remainder" %(chain,cyclesover)
						break
					i = i+1

			#draw to the last part of the patch
			j = 1
			while j <= cycles:
				one = j/2
				two = j%2
				if two == 0:
					factors = 1.0
				else:
					factors = -1.0

				l = 1
				while l <= chain:
					y = ybase - (l)*2.0*R*factors*fact*dir
					l1 = CD.endCD -k
					l2 = rev[k:k+1]
					loop_image_data2(r,CD,l1,l2,x,y,whichside,verbose)
					k = k+1
					l = l + 1

				n = 0
				offs = .578*R
				xstart = x
				ybase = y - 2.0*R*factors*fact*dir
				while n < 2:
					x = xstart -offs -n*2.0*R
					l1 = CD.endCD - k
					l2 = rev[k:k+1]
					loop_image_data2(r,CD,l1,l2,x,ybase,whichside,verbose)
					k = k+1
					n = n+1

				x = xstart - 3.15*R
				j = j+1

			dir = -1.0*dir
			#draw up to the feature section
			j = 1
			while j < cyclesover:
				y = ybase -j*2.0*R*factors*fact*dir
				l1 = CD.endCD - k
				l2 = rev[k:k+1]
				loop_image_data2(r,CD,l1,l2,x,y,whichside,verbose)
				k = k+1
				j = j+1
			
			print "y is %f" %y
			#draw the last part of the feature section
			patchc = r.cusend[cusidnum+2] - r.cusstart[cusidnum+2] + 1
			#print "patchc is %d" %patchc
			if patchc == max_patch:
				buff = 0.0
			else:
				buff = float(max_patch - patchc)/float(patchc + 1)

			#print "buff is %f " %buff
			#print "y begin is %f" %y
                        ystart = y 
			#print "ystart is %f" %ystart
			toppatch = ystart -2.0*R*factors*fact*dir*float(max_patch+1) 
			#print "toppatch is %f" %toppatch
			j = 1
	                while j <= patchc:
                                y = ystart -2.0*j*factors*fact*dir*R*(1.0+buff)
				print "y is %f" %y
				l1 = CD.endCD - k
				l2 = rev[k:k+1]
				loop_image_data2(r,CD,l1,l2,x,y,whichside,verbose)
                                j = j + 1
                                k = k + 1

			j = 1
			ystart = toppatch + 2.0*dir*fact*factors*R
	                while j <= leg4up:
                                y = ystart -2.0*j*R*fact*dir*factors 
				l1 = CD.endCD - k
				l2 = rev[k:k+1]
				loop_image_data2(r,CD,l1,l2,x,y,whichside,verbose)
                                j = j + 1
                                k = k + 1
			
			offs = .578*R
			if leg4upover > 0:
				nums = 2
			else:
				nums = 1
			if nums == 1:
				offs = 1.578*R

			n = 0
			y = y -2.0*R*fact*dir*factors
			while n < nums:
				l1 = CD.endCD - k
				l2 = rev[k:k+1]
				l3 = x - offs - n*2.0*R
				loop_image_data2(r,CD,l1,l2,l3,y,whichside,verbose)
                                k = k + 1
                                n = n + 1

			j = 1
			dir = -1.0*dir
			x = x - 3.15*R
			ystart = y
	                while j <= leg4up:
                                y = ystart -2.0*j*R*fact*dir*factors 
				l1 = CD.endCD - k
				l2 = rev[k:k+1]
				loop_image_data2(r,CD,l1,l2,x,y,whichside,verbose)
                                j = j + 1
                                k = k + 1

			# put in the next patch area
			patchb = r.cusend[cusidnum+1] - r.cusstart[cusidnum+1] + 1
			print "patchb is %d" %patchb
			if patchb == max_patch:
				buff = 0.0
			else:
				buff = float(max_patch - patchb)/float(patchb + 1)
                                       
			#print "buff is %f " %buff

			ystart = y
			j = 1
	                while j <= patchb:
				print "k is %d" %k
				
                                y = ystart -2.0*j*R*fact*dir*factors*(1.0+buff) 
				l1 = CD.endCD - k
				l2 = rev[k:k+1]
				loop_image_data2(r,CD,l1,l2,x,y,whichside,verbose)
                                j = j + 1
                                k = k + 1

			j = 1
			ystart = y
			#draw a tight loop or a long pointing one
			checklong0 = cyclesover - (max_patch+cycles)
			checklong = leg3x - 2*(cyclesover+2) 
			xref = x
			xref2 = x - 3.15*R
			if checklong >= 20:
				#print "cyclesover is %d checklong0 is %d leg3x is %d and checklong is %d" %(cyclesover,checklong0,leg3x,checklong)
				#draw down to the branch point
				#determine the size of the 4 areas
				branch = (checklong -2)/4
				branchover = (checklong - 2)%4
				#print "branch is %d branchover is %d" %(branch,branchover)
				if branchover == 3:
					cyclesover = cyclesover + 1
                       		while j <= cyclesover+2:
                               		y = ystart -2.0*j*R*fact*dir*factors
					l1 = CD.endCD -k
					l2 = rev[k:k+1]
                               		loop_image_data2(r,CD,l1,l2,x,y,whichside,verbose)
                               		j = j + 1
                               		k = k + 1

				yover = y-2.0*R*dir*fact*factors
                                n = 0
       	                        while n < branch:
               	                        l1 = CD.endCD -k
                       	                l2 = rev[k:k+1]
                               	        l3 = x + n*2.0*R
                                       	loop_image_data3(r,k,CD,l1,l2,l3,yover,whichside,verbose)
                                       	n = n + 1
                                       	k = k + 1

                               	offs = .578*R
                               	#draw the cross down section
				xstart = l3
                               	if branchover >= 1:
                                       	nums = 2
                               	else:
                                       	nums = 1
 
                               	if nums == 1:
                                       	offs = 1.578*R

                              	#print "nums is %d" %nums
				n = 0
				l3 = xstart + 2.0*R
                                while n < nums: 
	                               	y = yover -offs*factors*fact*dir -n*2.0*R*factors*fact*dir
                                       	l1 = CD.endCD -k
                                       	l2 = rev[k:k+1]
                                       	loop_image_data2(r,CD,l1,l2,l3,y,whichside,verbose)
                                       	k = k + 1
                                       	n = n + 1

				#branch back to middle
	                        n = 0
				y = yover -3.0*R*factors*fact*dir
        	                while n < branch:
                	        	l1 = CD.endCD -k
                        	        l2 = rev[k:k+1]
                                	l3 = xstart - n*2.0*R
                                        loop_image_data3(r,k,CD,l1,l2,l3,y,whichside,verbose)
                                        n = n + 1
                                        k = k + 1

				#second part of the branch
				x = xref2
				n = 0
        	                while n < branch:
                	        	l1 = CD.endCD -k
                        	        l2 = rev[k:k+1]
                                	l3 = x - n*2.0*R
                                        loop_image_data3(r,k,CD,l1,l2,l3,y,whichside,verbose)
                                        n = n + 1
                                        k = k + 1

                                offs = .5*R
                                #draw the cross up section
				xstart = l3
                                if branchover ==3:
				 	nums = 1 
				if branchover == 2:
                                       	nums = 2
				if branchover == 1:
                                        nums = 1
 
                                if nums == 1:
                                        offs = 1.5*R

                               	#print "nums is %d" %nums
				n = 0
				l3 = xstart - 2.0*R
				ystart = y
                                while n < nums: 
	                                y = ystart +offs*factors*fact*dir +n*2.0*R*factors*fact*dir
                                        l1 = CD.endCD -k
                                        l2 = rev[k:k+1]
                                        loop_image_data3(r,k,CD,l1,l2,l3,y,whichside,verbose)
                                        k = k + 1
                                        n = n + 1

				x = l3
				y = yover
	                        n = 0
        	                while n < branch:
                	        	l1 = CD.endCD -k
                        	        l2 = rev[k:k+1]
                                	l3 = x + (n+1)*2.0*R
                                        loop_image_data3(r,k,CD,l1,l2,l3,y,whichside,verbose)
                                        n = n + 1
                                        k = k + 1

				j = 1
				dir = -1.0*dir
				x = xref2
				ystart = yover
                        	while j <= cyclesover+2:
                                	y = ystart -2.0*j*R*fact*dir*factors
					l1 = CD.endCD -k
					l2 = rev[k:k+1]
                               		loop_image_data2(r,CD,l1,l2,x,y,whichside,verbose)
                                	j = j + 1
                                	k = k + 1


			#draw the first part of the feature section
			patcha = r.cusend[cusidnum] - r.cusstart[cusidnum] + 1
			print "patcha is %d" %patcha
			if patcha == max_patch:
				buff = 0.0
			else:
				buff = float(max_patch - patcha)/float(patcha + 1)
                                       
			#print "buff is %f " %buff
			leftpatch = x
                        numelem = r.cuselements[cusidnum]
                        makefeaturebox(r,whichside,leftpatch,toppatch,max_patch,numelem,verbose)
			ystart = y
			j = 1
	                while j <= patcha:
				print "k is %d" %k
				
                                y = ystart -2.0*j*R*fact*dir*factors*(1.0+buff) 
				l1 = CD.endCD - k
				l2 = rev[k:k+1]
				loop_image_data2(r,CD,l1,l2,x,y,whichside,verbose)
                                j = j + 1
                                k = k + 1

			#up to get to a good point to start repeating
			leg5 = chain - (cyclesover + max_patch -1)
			ystart = y -2.0*R*fact*dir*buff*factors
			j = 1
			while j <= leg5:
                                y = ystart -2.0*j*R*fact*dir*factors
				l1 = CD.endCD - k
				l2 = rev[k:k+1]
                                loop_image_data2(r,CD,l1,l2,x,y,whichside,verbose)
                                j = j + 1
                                k = k + 1

			print " k is %d" %k
	                offs = .5*R
                        #draw the cross section
                        nums = 2
                        n = 0
                        y = y -2.0*R*fact*dir*factors
                        while n < nums: 
                                l1 = CD.endCD -k
                                l2 = rev[k:k+1]
                                l3 = x - offs - n*2.0*R
				loop_image_data3(r,k,CD,l1,l2,l3,y,whichside,verbose)
                                k = k + 1
                                n = n + 1

                        x = x - 3.0*R
                        dir = -1.0*dir
			print " j is %d" %j


			#determine the rest of the sequence
                        y = y -2.0*R*fact*dir*factors

			resleft = CD.endCD -k
			lop = resleft/(chain + 1)
			lox = resleft%(chain+1.0)
			if lox > 0:
				lop = lop+1
			j = 1
			ystart = y
			#dir = -1.0*dir
			while j <= lop:
				print "y is %f in pass %d" %(y,j)

				l = 1
				while l <= chain:
					y = ystart - (l-1)*2.0*R*dir*fact*factors
					if CD.numCD == 0:
						l1 = CD.endCD - k
	        	                	l2 = rev[k:k+1]
						if l1 != 0:
							loop_image_data2(r,CD,l1,l2,x,y,whichside,verbose)

					l = l + 1
					if k == N:
						#print "ending this"
						#print "j is %d and k is %d" % (j,k)
						break
					k = k + 1
				if k == N:
					break

				#draw cross over section
				n = 0
				nums = 2

				y = y -2.0*R*dir*fact*factors
				offs = .5*R
				xstart = x 
				while n < nums:
					x = xstart - offs - n*2.0*R
					l1 = CD.endCD - k
	       	                	l2 = rev[k:k+1]
					if l1 != 0:
						loop_image_data2(r,CD,l1,l2,x,y,whichside,verbose)

					n = n + 1
					if k == N:
						break
					k = k +1
				
				dir = -1.0*dir
				x = xstart - 3.0*R
				y = y - 2.0*R*dir*factors*fact
				if k == N:
					break

				j = j + 1

				ystart = y

		else:
		#start to draw the repeating section
		#print "ybase is %f ybaseb is %f and ybaset is %f" % (ybase,ybaseb,ybaset)

			while j <= lop:
				#check to see which leg you are on
				one = j/2
				two = j%2
				if two == 0:
					factors = 1.0
				else:
					factors = -1.0

				if CD.numCD == 99:
					factors = -1.0*factors

				if whichside == 2:			
					ybase = y - 2.0*R

				l = 1
				while l <= chain:
					if whichside == 2:
						if j == 1:
							m = l-1
						else:
							m = l

					elif whichside == 1:
						if two != 0:
							m = l-1
							ybase = ybaseb
						elif two == 0:
							m = l
							ybase = ybaset
						
					y = ybase - (m)*2.0*R*fact*factors*dir
					if CD.numCD == 0:
						l1 = CD.endCD - k
	        	                	l2 = rev[k:k+1]
						if l1 != 0:
							loop_image_data2(r,CD,l1,l2,x,y,whichside,verbose)

					else:
						l1 = CD.startCD + k
                                		l2 = CD.strandCD[k:k+1]
						loop_image_data3(r,k,CD,l1,l2,x,y,whichside,verbose)

					l = l + 1
					if k == N:
						#print "ending this"
						#print "j is %d and k is %d" % (j,k)
						break
					k = k + 1
				if k == N:
					break

				#draw cross over section
				n = 1
				if CD.numCD == 99:
					x = x + R
				else:
					x = x - R

				while n <=2:
					if j == 1:
						m = chain
					else:
						m = chain+1

					if whichside == 1:
						if two != 0:				
							m = 0
							ybase = ybaset
						else:
							m = 1
							ybase = ybaseb

					y = ybase - (m)*2.0*R*factors*fact*dir
					if CD.numCD == 0:
						l1 = CD.endCD - k
	        	                	l2 = rev[k:k+1]
						if l1 != 0:
							loop_image_data2(r,CD,l1,l2,x,y,whichside,verbose)
					else:
						l1 = CD.startCD + k
       	                        		l2 = CD.strandCD[k:k+1]
						loop_image_data3(r,k,CD,l1,l2,x,y,whichside,verbose)
					n = n + 1
					if k == N:
						break
					k = k +1
				
					if CD.numCD == 99:
						x = x + 2.0*R
						y = y + 2.0*R 
					else:
						x = x - 2.0*R
						y = y + 2.0*R
				if k == N:
					break

				j = j + 1
				if CD.numCD == 99:
					x = x - R
				else:
					x = x + R

	elif CD.numCD > 0 and CD.numCD < 99:
		ystart = ybase
		if N <= 9:
			div = 0.0
			if N <= 5:
				div = W/(N+1)
			else:
				div = W/N
			#print "w is %d and div is %d" % (W,div)
			j = 1
			k = 0
			out = div - R
			while j <= N:
				l1 = CD.startCD +k
				l2 = CD.strandCD[k:k+1]
				if N == 9:
					l3 = x + j*2.0*R - 2.0*R
				else:
					l3 = x + out + (j-1)*div
				if l1 != 0:
					loop_image_data2(r,CD,l1,l2,l3,ystart,whichside,verbose)
				j = j + 1
				k = k + 1
				
		elif N > 9 and N <= 23: 
			#determine the length of possible sides
			side = (N-7)/2
			odd = (N-7)%2

			#if odd increase side by 1 and reduce cross section by 1
			if odd > 0:
				side = side + 1

			#draw first side
 			j = 1
			k = 0
			while j <= side:
				y = ystart -2.0*(j - 1)*R*fact	
				loop_image_data(r,k,CD,x,y,whichside,verbose)
				j = j + 1
				k = k + 1

			#draw the cross section
			#print "odd is %d" % odd
			if odd > 0:
				num = 6
				off = .286*R
				#print "offset is %d" % off
			else:
				num = 7
				off = 0.0

			j = 1
			xstart = x
			while j <= num:
                        	x = xstart + j*(2.0*R + off)
				loop_image_data(r,k,CD,x,y,whichside,verbose)
                        	j = j + 1
                       		k = k + 1 

			#draw the other side
			x = x + 2.0*R + off
 			j = 1
			ystart = y
			while j <= side:
				y = ystart +2.0*(j-1)*R*fact			
				#print "y is %f and k is %d" % (y,k)
				loop_image_data(r,k,CD,x,y,whichside,verbose)
				j = j + 1
				k = k + 1

		elif N > 23 and N < 63:
			#determine the length of possible sides
			side = (N-9)/6
			leftover = (N-9)%6

			#draw structure
			i = 1
 			j = 1
			k = 0
			n = 0
			#put in the base
			while n < 2:
				y = ystart -2.0*n*R*fact	
				loop_image_data(r,k,CD,x,y,whichside,verbose)
				n = n + 1
				k = k + 1

			#draw the repeat and caps
			ystart = y
			dir = 1.0
			while i <= 6: 
				while j <= side:
					y = ystart -2.0*j*R*fact*dir	
					loop_image_data(r,k,CD,x,y,whichside,verbose)
					j = j + 1
					k = k + 1

				#draw the cross section
				if leftover > i - 1:
					nums = 2
					offs = .6*R
				else:
					nums = 1
					offs = 1.6*R

				n = 0
				y = y -2.0*R*fact*dir
				while n < nums: 
					if k == N -2:
						break
					l1 = CD.startCD +k
					l2 = CD.strandCD[k:k+1]
					l3 = x + offs + n*2.0*R
					loop_image_data3(r,k,CD,l1,l2,l3,y,whichside,verbose)
					k = k + 1
					n = n + 1
				
				if k == N - 2:
					break
				x = x + 3.2*R
				ystart = y
				dir = -1.0*dir
				i = i + 1
				j = 1
			#draw the other base
			n = 0
			while n < 2:
				y = y -2.0*n*R*fact*dir	
				loop_image_data(r,k,CD,x,y,whichside,verbose)
				n = n + 1
				k = k + 1

		elif N >= 63:
			#check to determine the type of structure to be drawn
			#print "got a biggie"

			# determine the number of loops
			num_loops = len(r.loops)
			#print "number of loops is %d " % num_loops
			p = 0
			ref_num = 0
			big = []
			while p < num_loops-1:
				longs_loop = r.loops[p].endLoop -r.loops[p].startLoop
				if longs_loop >= 101:
					#print " loop %d is %d residues long" %(p+1,longs_loop)
					big.append(p)
					ref_num = ref_num + 1
				p = p + 1

			#print "ref_num is %d" % ref_num
			pz = 0
			bull = 1

			if custom == 1:
				msg = "this is a custom loop"
				printVerbose(verbose,msg)

			while pz <= ref_num-1:
				nums = big[pz]
				ref_long =  r.loops[nums].endLoop -r.loops[nums].startLoop + 1
				if ref_long == N:
					bull = 0
					#print "checking for stuff"
					# check for other close loops on the same side
					myside = r.loops[nums].sideLoop
					#print "myside = %d " % myside
					if myside == r.loops[nums-1].sideLoop:
						#print "into checks off 1"
						l_loop = r.loops[nums-1].endLoop - r.loops[nums-1].startLoop
						#print "l_loop is %d" % l_loop
						if l_loop > 101:
							bull = bull + 1
					elif myside == r.loops[nums-2].sideLoop:
						#print "into checks off 2"
						l_loop = r.loops[nums-2].endLoop - r.loops[nums-2].startLoop
						#print "l_loop is %d" % l_loop
						if l_loop > 101:
							bull = bull + 1
						#print "bull is %d" % bull
			
					# check for too small a loop to make the structure
					ref_loop = r.loops[nums].endLoop - r.loops[nums].startLoop
					if ref_loop < 91:
						bull = 1
				else:
					bull = 1
				pz = pz + 1

			#check for a problem with the ending section for a bullwinkle
			if bull == 0 and ref_num > num_loops -2:
				end_prob = 1

			#print "bull is %d" % bull
			#print "custom is %d" %custom
			if bull == 0 and custom == 1:
                        	#print "can do a custom bullwinkle"
				buff = 0.0
				if whichside == 1:
					dir = 1.0
                                #determine the length of possible sides
                                side0 = (N-75)/20
                                leftover0 = (N-75)%20
                                #print "leftover is %d"% leftover
                                #draw structure until the loops start
                                i = 1
                                j = 1
                                k = 0
                                n = 0
                                #put in the base on the left
                                while n < 15:
                                        y = ystart -2.0*n*R*dir*fact
                                        loop_image_data(r,k,CD,x,y,whichside,verbose)
                                        n = n + 1
                                        k = k + 1

                                #over to the left more
                                n = 0
                                while n < 11:
                                        l1 = CD.startCD +k
                                        l2 = CD.strandCD[k:k+1]
                                        l3 = x - (n+1)*2.0*R
                                        loop_image_data3(r,k,CD,l1,l2,l3,y,whichside,verbose)
                                        n = n + 1
                                        k = k + 1

				#print info
				#print "whichside is %d" %whichside
                                #down two more
                                ystart = y
                                x = l3
                                n = 1
                                while n <= 2:
                                        y = ystart -2.0*n*R*dir*fact
                                        loop_image_data(r,k,CD,x,y,whichside,verbose)
                                        n = n + 1
                                        k = k + 1

				#determine the variables
				ystart = y
				yref_top = y
				curr_loc = CD.startCD + 28

				ij = 0
				max_patch = 0
				while ij < r.cuselements[cusidnum]:
					if cusidnum == 0:
						idoff = 0
					else:
						idoff = 3.0*(cusidnum - 1)
					patch_size = r.cusend[ij + idoff] - r.cusstart[ij + idoff] + 1
					if patch_size > max_patch:
						max_patch = patch_size
					#print "ij is %d and patch size is %d"  %(ij,patch_size)
					ij = ij + 1

				leg2 = r.cusstart[cusidnum+1] - r.cusend[cusidnum] -1
				leg2up = (leg2-1)/2
				leg2upover = (leg2-1)%2
				#print "max patch is %d leg2 is %d leg2up is %d and leg2upover is %d" %(max_patch,leg2,leg2up,leg2upover)

				leg3 = r.cusstart[cusidnum+2] - r.cusend[cusidnum+1] -1
				leg3up = (leg3-1)/2
				leg3upover = (leg3-1)%2
				#print "max patch is %d leg3 is %d leg3up is %d and leg3upover is %d" %(max_patch,leg3,leg3up,leg3upover)

				ntimes = 2
				leg1 = r.cusstart[cusidnum] - curr_loc -(1+leg3up)
				leg1up = (leg1-1)/2
				leg1upover = (leg1-1)%2
				#print "max patch is %d leg1 is %d leg1up is %d and leg1upover is %d" %(max_patch,leg1,leg1up,leg1upover)
				if leg1up > 30:
					leg1up = (leg1-3)/4
					leg1upover = (leg1-3)%4
					#print "max patch is %d leg1 is %d leg1up is %d and leg1upover is %d" %(max_patch,leg1,leg1up,leg1upover)
					ntimes = 4
				
				#path one (patch in 3rd chain)
				#print "this part needs work"

                        	#draw the determined loops and caps
                        	i = 1
                        	j = 1
                        	while i <= ntimes:
                                	while j <= leg1up:
                                        	y = ystart -2.0*j*R*dir*fact
                                        	loop_image_data(r,k,CD,x,y,whichside,verbose)
                                        	j = j + 1
                                        	k = k + 1
                                	offs = .578*R
                                	#draw the cross section
                                	if leg1upover > i - 1:
                                        	nums = 2
                                	else:
                                        	nums = 1
 
                                	if nums == 1:
                                        	offs = 1.578*R
                                	n = 0
                                	y = y -2.0*R*dir*fact
                                	while n < nums: 
                                        	l1 = CD.startCD +k
                                        	l2 = CD.strandCD[k:k+1]
                                        	l3 = x + offs + n*2.0*R
                                        	loop_image_data3(r,k,CD,l1,l2,l3,y,whichside,verbose)
                                        	k = k + 1
                                        	n = n + 1

                                	x = x + 3.15*R
                                	ystart = y
                                	dir = -1.0*dir
                                	i = i + 1
                                	j = 1

                        	#draw down to the start of patch
                        	j = 1
                        	while j <= leg3up:
                                	y = ystart -2.0*j*R*dir*fact
                                	loop_image_data(r,k,CD,x,y,whichside,verbose)
                                	j = j + 1
                                	k = k + 1

				#draw patch area
				toppatch = y
				leftpatch = x
				numelem = r.cuselements[cusidnum]
				makefeaturebox(r,whichside,leftpatch,toppatch,max_patch,numelem,verbose)
				ystart = y
				j = 1
				patcha = r.cusend[cusidnum]-r.cusstart[cusidnum] + 1
				if patcha == max_patch:
					buff = 0.0
				else:
					buff = float(max_patch - patcha)/float(patcha + 1)                                        

	                	while j <= patcha:
                                	y = ystart -2.0*j*R*dir*fact*(1.0+buff)
                                	loop_image_data(r,k,CD,x,y,whichside,verbose)
                                	j = j + 1
                                	k = k + 1

				botpatch = y
				ystart = y
				j = 1
				#draw a tight loop or a long pointing one
				checklong0 = (leg1up+1) - (leg3up + max_patch)
				checklong = leg2 - 2*(checklong0+2) 
				xref = x
				xref2 = x + 3.15*R
				if checklong >= 20:
					#print "checklong0 is %d and checklong is %d" %(checklong0,checklong)
					#draw down to the branch point
                        		while j <= checklong0+2:
                                		y = ystart -2.0*j*R*dir*fact
                                		loop_image_data(r,k,CD,x,y,whichside,verbose)
                                		j = j + 1
                                		k = k + 1

					#determine the size of the 4 areas
					branch = (checklong -2)/4
					branchover = (checklong - 2)%4
					#print "branchover for middle section is %d" %branchover
					yover = y-2.0*R*dir*fact
	                                n = 0
        	                        while n < branch:
                	                        l1 = CD.startCD +k
                        	                l2 = CD.strandCD[k:k+1]
                                	        l3 = x - n*2.0*R
                                        	loop_image_data3(r,k,CD,l1,l2,l3,yover,whichside,verbose)
                                        	n = n + 1
                                        	k = k + 1

                                	offs = .578*R
                                	#draw the cross down section
					xstart = l3
                                	if branchover >= 1:
                                        	nums = 2
                                	else:
                                        	nums = 1
 
                                	if nums == 1:
                                        	offs = 1.578*R

                               		#print "nums is %d" %nums
				 	n = 0
					l3 = xstart - 2.0*R
                                	while n < nums: 
	                                	y = yover -offs*dir*fact -n*2.0*R*dir*fact
                                        	l1 = CD.startCD +k
                                        	l2 = CD.strandCD[k:k+1]
                                        	loop_image_data3(r,k,CD,l1,l2,l3,y,whichside,verbose)
                                        	k = k + 1
                                        	n = n + 1

					#branch back to middle
	                                n = 0
					y = yover -3.15*R*dir*fact
        	                        while n < branch:
                	                        l1 = CD.startCD +k
                        	                l2 = CD.strandCD[k:k+1]
                                	        l3 = xstart + n*2.0*R
                                        	loop_image_data3(r,k,CD,l1,l2,l3,y,whichside,verbose)
                                        	n = n + 1
                                        	k = k + 1

					#second part of the branch
					x = xref2
					n = 0
        	                        while n < branch:
                	                        l1 = CD.startCD +k
                        	                l2 = CD.strandCD[k:k+1]
                                	        l3 = x + n*2.0*R
                                        	loop_image_data3(r,k,CD,l1,l2,l3,y,whichside,verbose)
                                        	n = n + 1
                                        	k = k + 1

                                	offs = .578*R
                                	#draw the cross up section
					xstart = l3
                                	if branchover > 2 or branchover == 2:
                                        	nums = 2
                                	else:
                                        	nums = 1
 
                                	if nums == 1:
                                        	offs = 1.578*R

                               		#print "nums is %d" %nums
					dir = -1.0*dir
				 	n = 0
					l3 = xstart + 2.0*R
					ystart = y
                                	while n < nums: 
	                                	y = ystart -offs*dir*fact -n*2.0*R*dir*fact
                                        	l1 = CD.startCD +k
                                        	l2 = CD.strandCD[k:k+1]
                                        	loop_image_data3(r,k,CD,l1,l2,l3,y,whichside,verbose)
                                        	k = k + 1
                                        	n = n + 1

					x = l3
					y = yover
	                                n = 0
        	                        while n < branch:
                	                        l1 = CD.startCD +k
                        	                l2 = CD.strandCD[k:k+1]
                                	        l3 = x - (n+1)*2.0*R
                                        	loop_image_data3(r,k,CD,l1,l2,l3,y,whichside,verbose)
                                        	n = n + 1
                                        	k = k + 1

					j = 1
					dir = -1.0*dir
					x = xref2
					ystart = yover
                        		while j <= checklong0+2:
                                		y = ystart +2.0*j*R*dir*fact
                                		loop_image_data(r,k,CD,x,y,whichside,verbose)
                                		j = j + 1
                                		k = k + 1


				else:
				#draw half of the loop to the next patch element					
                        		while j <= leg2up:
                                		y = ystart -2.0*j*R*dir*fact
                                		loop_image_data(r,k,CD,x,y,whichside,verbose)
                                		j = j + 1
                                		k = k + 1


					offs = .578*R
                        		#draw the cross section
                        		if leg2upover > 0:
						nums = 2
					else:
						nums = 1
 					if nums == 1:
						offs = 1.578*R

                        		n = 0
                       			y = y -2.0*R*dir
                        		while n < nums: 
                        			l1 = CD.startCD +k
                                		l2 = CD.strandCD[k:k+1]
                                		l3 = x + offs + n*2.0*R
                                		loop_image_data3(r,k,CD,l1,l2,l3,y,whichside,verbose)
                                		if k == N - 28:
                                			break
                                		k  = k + 1
                                		n = n + 1

                        		x = x + 3.15*R
                        		ystart = y
                        		dir = -1.0*dir
                        		j = 1

					#draw 2nd half of the loop to the next patch element					
                        		while j <= leg2up:
                                		y = ystart -2.0*j*R*dir*fact
                                		loop_image_data(r,k,CD,x,y,whichside,verbose)
                                		j = j + 1
                                		k = k + 1

				patchb = r.cusend[cusidnum+1]-r.cusstart[cusidnum+1] + 1
				if patchb == max_patch:
					buff = 0.0
				else:
					buff = float(max_patch - patchb)/float(patchb+1)                                        

				ystart = y
				j = 1
	                	while j <= patchb:
                                	y = ystart +2.0*j*R*dir*fact*(1.0+buff)
                                	loop_image_data(r,k,CD,x,y,whichside,verbose)
                                	j = j + 1
                                	k = k + 1

				ystart = y
				j = 1
                        	while j <= leg3up:
                                	y = ystart +2.0*j*R*dir*fact
                                	loop_image_data(r,k,CD,x,y,whichside,verbose)
                                	j = j + 1
                                	k = k + 1

				offs = .578*R
                        	#draw the cross section
                        	if leg3upover > 0:
					nums = 2
				else:
					nums = 1
 				if nums == 1:
					offs = 1.578*R

                        	n = 0
                       		y = y +2.0*R*dir*fact
                        	while n < nums: 
                                	l1 = CD.startCD +k
                                	l2 = CD.strandCD[k:k+1]
                                	l3 = x + offs + n*2.0*R
                                	loop_image_data3(r,k,CD,l1,l2,l3,y,whichside,verbose)
                                	if k == N - 28:
                                        	break
                                	k = k + 1
                                	n = n + 1

                          	x = x + 3.15*R
                          	ystart = y
                          	dir = -1.0*dir
                          	j = 1
                          	while j <= leg3up:
                                	y = ystart +2.0*j*R*dir*fact
                                	loop_image_data(r,k,CD,x,y,whichside,verbose)
                                	j = j + 1
                                	k = k + 1

				patchc = r.cusend[cusidnum+2] - r.cusstart[cusidnum+2] + 1
				#print "patchc is %d" %patchc
				if patchc == max_patch:
					buff = 0.0
				else:
					buff = float(max_patch - patchc)/float(patchc + 1)
                                        
				#print "buff is %f " %buff
				ystart = y
				j = 1
	                	while j <= patchc:
                                	y = ystart +2.0*j*dir*R*fact*(1.0+buff) 
                                	loop_image_data(r,k,CD,x,y,whichside,verbose)
                                	j = j + 1
                                	k = k + 1

				ystart = y+2.0*R*dir*buff*fact
				#check to make a proper size down section (not too big)
				jcheck = (yref_top-ystart)/10
				if jcheck < 0:
					jcheck = -1*jcheck
				jdown = leg1up-jcheck
				#print "jcheck is %d, jdown is %d and leg1up is %d" %(jcheck,jdown,leg1up)
				if jdown <= 0:
					y = ystart
				if jdown > max_patch:
					jdown = patchc
				j = 1
				#draw connection to return to cycling area
				if jdown > 0:
	                        	while j <= jdown + 1:
        	                        	y = ystart +2.0*j*R*dir*fact 
                	                	loop_image_data(r,k,CD,x,y,whichside,verbose)
                        	        	j = j + 1
                                		k = k + 1

				yref_bot = y
                        	offs = .578*R
				nums = 2
                        	n = 0
                        	y = y +2.0*R*dir*fact
                        	while n < nums:  
                                	l1 = CD.startCD +k
                                	l2 = CD.strandCD[k:k+1]
                                	l3 = x + offs + n*2.0*R
                                	loop_image_data3(r,k,CD,l1,l2,l3,y,whichside,verbose)
                                	if k == N - 28:
                                        	break
                                	k = k + 1
                                	n = n + 1
                                
				x = x + 3.15*R
                        	ystart = y
                        	dir = -1.0*dir
                        	j = 1
				#determine the number of symbols back to good starting point
				numsym = (yref_top -yref_bot)/10
				if numsym < 0:
					numsym = -1*numsym
				#print "numsym is %d" %numsym
                        	while j <= numsym:
                                	y = ystart +2.0*j*R*dir*fact
                                	loop_image_data(r,k,CD,x,y,whichside,verbose)
                                	j = j + 1
                                	k = k + 1

				#put in caps
                        	offs = .578*R
                        	nums = 2
                       		n = 0
                        	y = y +2.0*R*dir*fact
                        	while n < nums:   
                                	l1 = CD.startCD +k
                               		l2 = CD.strandCD[k:k+1]
                                 	l3 = x + offs + n*2.0*R
                                 	loop_image_data3(r,k,CD,l1,l2,l3,y,whichside,verbose)
                                 	if k == N - 28:
                                        	break
                                 	k = k + 1
                                 	n = n + 1
                                        
                      		x = x + 3.15*R
                        	ystart = y
                        	dir = -1.0*dir
                        	j = 1

				#determine the number of loops and cycles
				resleft = CD.clenCD - k - 26  
				cyclesleft = 20 - (ntimes + r.cuselements[cusidnum] +1)
				#print "there are %d residues in looping section left and %d loops" %(resleft,cyclesleft)
				sidenew = (resleft - cyclesleft - 1)/cyclesleft
				#print "sidenew is %d" %sidenew
				sidenewover = (resleft - cyclesleft - 1)%cyclesleft
	                	#draw the repeat and caps
				i = 1
                        	while i <= cyclesleft: 
                        		while j <= sidenew:
                                		y = ystart +2.0*j*R*dir*fact
                                        	loop_image_data(r,k,CD,x,y,whichside,verbose)
                                        	j = j + 1
                                        	k = k + 1

                               		offs = .578*R
                                	#draw the cross section
                                	if sidenewover > i - 1:
                                		nums = 2
                                	else:
                                        	nums = 1
                                        	offs = 1.578*R

                                	n = 0
                                	y = y +2.0*R*dir*fact
                                	if k == N - 28:
                                		break
                                	while n < nums: 
                                		l1 = CD.startCD +k
                                        	l2 = CD.strandCD[k:k+1]
                                        	l3 = x + offs + n*2.0*R
                                        	loop_image_data3(r,k,CD,l1,l2,l3,y,whichside,verbose)
                                        	if k == N - 28:
                                        		break
                                        	k = k + 1
                                       		n = n + 1
		
					x = x + 3.15*R
                        		ystart = y
                                	dir = -1.0*dir
                                	i = i + 1
					j = 1

				#finish off the loop
		        	#up two more
                		ystart = y
                        	n = 0
                        	while n < 2:
                        		y = ystart +2.0*n*R*dir*fact
                                	loop_image_data(r,k,CD,x,y,whichside,verbose)
                                	n = n + 1
                                	k = k + 1
                       		#back 11 to the left
                        	n = 0
                        	ystart = y
                        	while n < 11:
                        		l1 = CD.startCD +k
                                	l2 = CD.strandCD[k:k+1]
                                	l3 = x - n*2.0*R
                                	l4 = ystart +2.0*R*dir*fact 
                                	loop_image_data3(r,k,CD,l1,l2,l3,l4,whichside,verbose)
                                	n = n + 1
                               		k = k + 1

				#back to the membrane
                        	x =l3 -2.0*R
                        	ystart = l4
                        	n = 0
                        	while n < 15:
                        		y = ystart +2.0*n*R*dir*fact
                                	loop_image_data(r,k,CD,x,y,whichside,verbose)
                                	n = n + 1
                                	k = k + 1

			elif bull == 0 and custom == 0:
				#print "can do a bullwinkle"
                               
				#determine the length of possible sides
				side = (N-75)/20
				leftover = (N-75)%20
				#print "leftover is %d"% leftover
				#draw structure
				i = 1
 				j = 1
				k = 0
				n = 0
				#put in the base on the left
				while n < 15:
					y = ystart -2.0*n*R*fact	
					loop_image_data(r,k,CD,x,y,whichside,verbose)
					n = n + 1
					k = k + 1

				#over to the left more
				n = 0
				while n < 11:
					l1 = CD.startCD +k
                               		l2 = CD.strandCD[k:k+1]
                               		l3 = x - (n+1)*2.0*R
					loop_image_data3(r,k,CD,l1,l2,l3,y,whichside,verbose)
					n = n + 1
					k = k + 1

				#down two more
				ystart = y
				x = l3
				n = 1
				while n <= 2:
					y = ystart -2.0*n*R*fact	
					loop_image_data(r,k,CD,x,y,whichside,verbose)
					n = n + 1
					k = k + 1

				#draw the repeat and caps
				ystart = y
				dir = 1.0
				while i <= 20: 
					while j <= side:
						y = ystart -2.0*j*R*fact*dir	
						loop_image_data(r,k,CD,x,y,whichside,verbose)
						j = j + 1
						k = k + 1

					offs = .578*R
					#draw the cross section
					if leftover > i - 1:
						nums = 2
					else:
						nums = 1
						offs = 1.578*R

					n = 0
					y = y -2.0*R*fact*dir
					if k == N - 28:
						break
					while n < nums: 
						l1 = CD.startCD +k
						l2 = CD.strandCD[k:k+1]
						l3 = x + offs + n*2.0*R
						loop_image_data3(r,k,CD,l1,l2,l3,y,whichside,verbose)
						if k == N - 28:
							break
						k = k + 1
						n = n + 1
				
					x = x + 3.15*R
					ystart = y
					dir = -1.0*dir
					i = i + 1
					j = 1

				#up two more
				ystart = y
				n = 0
				while n < 2:
					y = ystart +2.0*n*R*fact	
					loop_image_data(r,k,CD,x,y,whichside,verbose)
					n = n + 1
					k = k + 1
				#back 11 to the left
				n = 0
				ystart = y
				r.farend = x
				msg = "r farend is %f" %r.farend
                                printVerbose(verbose,msg)
				while n < 11:
					l1 = CD.startCD +k
                               		l2 = CD.strandCD[k:k+1]
                               		l3 = x - n*2.0*R
                               		l4 = ystart +2.0*R*fact 
					loop_image_data3(r,k,CD,l1,l2,l3,l4,whichside,verbose)
					n = n + 1
					k = k + 1

				#back to the membrane
				x =l3 -2.0*R
				ystart = l4
				n = 0
				while n < 15:
					y = ystart -2.0*n*R*fact*dir	
					loop_image_data(r,k,CD,x,y,whichside,verbose)
					n = n + 1
					k = k + 1

			else:
				# regular long structure with no overlap
				#determine the length of possible sides
				side = (N-23)/10
				leftover = (N-23)%10
				#print "start is %d and end is %d" % (CD.startCD,CD.endCD)
				#print "leftover is %d" % leftover

				#draw structure
				i = 1
 				j = 1
				k = 0
				n = 1
				#put in the base left
				while n <= 2:
					y = ystart -2.0*(n-1)*R*fact	
					loop_image_data(r,k,CD,x,y,whichside,verbose)
					n = n + 1
					k = k + 1

				#over to the left more
				n = 0
				while n < 3:
					l1 = CD.startCD +k
                               		l2 = CD.strandCD[k:k+1]
                               		l3 = x - (n+1)*2.0*R
					loop_image_data3(r,k,CD,l1,l2,l3,y,whichside,verbose)
					n = n + 1
					k = k + 1

				#down two more
				ystart = y
				x = l3
				n = 1
				while n <= 2:
					y = ystart -2.0*n*R*fact	
					loop_image_data(r,k,CD,x,y,whichside,verbose)
					n = n + 1
					k = k + 1

				#draw the repeat and caps
				ystart = y
				dir = 1.0
				while i <= 10: 
					while j <= side:
						y = ystart -2.0*j*R*fact*dir	
						loop_image_data(r,k,CD,x,y,whichside,verbose)
						j = j + 1
						k = k + 1

					offs = .55*R
					#draw the cross section
					if i == 1 and leftover > 0:
						nums = 2
					elif i == 2 and leftover > 1:
						nums = 2
					elif i == 3 and leftover > 2:
						nums = 2
					elif i == 4 and leftover > 3:
						nums = 2
					elif i == 5 and leftover > 4:
						nums = 2
					elif i == 6 and leftover > 5:
						nums = 2
					elif i == 7 and leftover > 6:
						nums = 2
					elif i == 8 and leftover > 7:
						nums = 2
					elif i == 9 and leftover > 8:
						nums = 2
 					else:
						nums = 1
						offs = 1.55*R

					n = 0
					y = y -2.0*R*fact*dir
					if k == N - 7:
						y = ystart -2.0*j*R*fact*dir	
						break
					while n < nums: 
						l1 = CD.startCD +k
						l2 = CD.strandCD[k:k+1]
						l3 = x + offs + n*2.0*R
						loop_image_data3(r,k,CD,l1,l2,l3,y,whichside,verbose)
						if k == N -7:
							break
						k = k + 1
						n = n + 1
				
					x = x + 3.11*R
					ystart = y
					dir = -1.0*dir
					i = i + 1
					j = 1

				#draw the other base
				#up two more
				ystart = y
				n = 0
				while n < 2:
					y = ystart +2.0*n*R*fact	
					loop_image_data(r,k,CD,x,y,whichside,verbose)
					n = n + 1
					k = k + 1
				#back three to the left
				n = 0
				ystart = y
				while n < 3:
					l1 = CD.startCD +k
                               		l2 = CD.strandCD[k:k+1]
                               		l3 = x - n*2.0*R
                               		l4 = ystart +2.0*R*fact 
					loop_image_data2(r,CD,l1,l2,l3,l4,whichside,verbose)
					n = n + 1
					k = k + 1

				#back to the membrane
				x =l3 -2.0*R
				ystart = l4
				n = 0
				while n < 2:
					y = ystart -2.0*n*R*fact*dir	
					loop_image_data(r,k,CD,x,y,whichside,verbose)
					n = n + 1
					k = k + 1


	return 

def loopcolors(r,k,whichside,CD,l1,l2,verbose):
	l5 = 1
	l6 = 'x'
	if r.cysdo == 'y' and whichside == 1:
		if l2 == 'C':
			l5 = r.cyssym		
			l6 = r.cyscol

	if r.glydo == 'y' and whichside == 1:
		if l2 == 'N':
			lx = CD.strandCD[k+2:k+3]
			if lx != '':
				msg = " checking position %d character %s downstream is %s" %(l1,l2,lx)
				printVerbose(verbose,msg)
				if lx == 'S' or lx == 'T':
					l5 = r.glysym		
					l6 = r.glycol
					msg= " found one at  %d %s" % (l1,lx)
					printVerbose(verbose,msg)

	return(l5,l6)

def loopcolors2(r,whichside,CD,l1,l2,verbose):
	l5 = 1
	l6 = 'x'
	if r.cysdo == 'y' and whichside == 1:
		if l2 == 'C':
			l5 = r.cyssym		
			l6 = r.cyscol

	lx = ''
	j = 0
	j2 = 0
	if r.glydo == 'y' and whichside == 1:
		if l2 == 'N':
			j = l1+1
			j2 = l1+2
			lx = CD.strandCD[j:j2]
			if lx != '':
				msg= " checking position %d character %s downstream is %s" %(l1,l2,lx)
				printVerbose(verbose,msg)
				if lx == 'S' or lx == 'T':
					l5 = r.glysym		
					l6 = r.glycol
					msg =  " found one at  %d %s" % (l1,lx)
					printVerbose(verbose,msg)

	return(l5,l6)


def loop_image_data(r,k,CD,x,y,whichside,verbose):

        l1 = CD.startCD + k
        l2 = CD.strandCD[k:k+1]
        l3 = x
        l4 = y
        lcol = loopcolors(r,k,whichside,CD,l1,l2,verbose)
        l5 = lcol[0]
        l6 = lcol[1]
        line = "%d %s %f %f %d %s" %(l1,l2,l3,l4,l5,l6)
	#print "loop_image_data line is %s" %line 
        linex = Image(line)
        r.image.append(linex)

        return

def loop_image_data2(r,CD,l1,l2,l3,l4,whichside,verbose):

        lcol = loopcolors2(r,whichside,CD,l1,l2,verbose)
        l5 = lcol[0]
        l6 = lcol[1]
        line = "%d %s %f %f %d %s" %(l1,l2,l3,l4,l5,l6)
	#print "loop_image_data2 line is %s" %line 
        linex = Image(line)
        r.image.append(linex)

        return

def loop_image_data3(r,k,CD,l1,l2,l3,l4,whichside,verbose):

        lcol = loopcolors(r,k,whichside,CD,l1,l2,verbose)
        l5 = lcol[0]
        l6 = lcol[1]
        line = "%d %s %f %f %d %s" %(l1,l2,l3,l4,l5,l6)
	#print "loop_image_data3 line is %s" %line 
        linex = Image(line)
        r.image.append(linex)

        return
